@@ -6,13 +6,23 @@ import java.io.IOException;
 import java.math.BigInteger;
 import java.security.PublicKey;
 import java.text.SimpleDateFormat;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.Date;
+import java.util.Enumeration;
+import java.util.HashSet;
+import java.util.Hashtable;
+import java.util.Iterator;
+import java.util.List;
+import java.util.Set;
 
+import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.ASN1OctetString;
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1TaggedObject;
 import org.bouncycastle.asn1.DERGeneralizedTime;
-import org.bouncycastle.asn1.DERInputStream;
 import org.bouncycastle.asn1.DERObject;
 import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.DEROutputStream;
@@ -23,7 +33,6 @@ import org.bouncycastle.asn1.x509.KeyPurposeId;
 import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.asn1.x509.X509Extensions;
 import org.bouncycastle.asn1.x509.X509Name;
-
 import org.bouncycastle.jce.PrincipalUtil;
 
 /**
@@ -62,7 +71,7 @@ import org.bouncycastle.jce.PrincipalUtil;
  * <b>TODO: implement name constraints</b>
  * <b>TODO: implement match check for path to names</b><br />
  * <br />
- * Uses {@link org.bouncycastle.asn1.DERInputStream DERInputStream},
+ * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
  * {@link org.bouncycastle.asn1.ASN1Sequence ASN1Sequence},
  * {@link org.bouncycastle.asn1.DERObjectIdentifier DERObjectIdentifier},
  * {@link org.bouncycastle.asn1.DEROutputStream DEROutputStream},
@@ -205,9 +214,9 @@ public class X509CertSelector implements CertSelector
      *   Name ::= CHOICE {
      *     RDNSequence }
      *
-     *   RDNSequence ::= SEQUENCE OF RelativeDistinguishedName
+     *   RDNSequence ::= SEQUENCE OF RDN
      *
-     *   RelativeDistinguishedName ::=
+     *   RDN ::=
      *     SET SIZE (1 .. MAX) OF AttributeTypeAndValue
      *
      *   AttributeTypeAndValue ::= SEQUENCE {
@@ -229,7 +238,7 @@ public class X509CertSelector implements CertSelector
      * Note that the byte array specified here is cloned to
      * protect against subsequent modifications.<br />
      * <br />
-     * Uses {@link org.bouncycastle.asn1.DERInputStream DERInputStream},
+     * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
      * {@link org.bouncycastle.asn1.DERObject DERObject},
      * {@link org.bouncycastle.asn1.ASN1Sequence ASN1Sequence},
      * {@link org.bouncycastle.asn1.x509.X509Name X509Name}
@@ -251,7 +260,7 @@ public class X509CertSelector implements CertSelector
         else
         {
             ByteArrayInputStream inStream = new ByteArrayInputStream(issuerDN);
-            DERInputStream derInStream = new DERInputStream(inStream);
+            ASN1InputStream derInStream = new ASN1InputStream(inStream);
             DERObject obj = derInStream.readObject();
             if ( obj instanceof ASN1Sequence )
             {
@@ -317,7 +326,7 @@ public class X509CertSelector implements CertSelector
      * ASN.1 notation for this structure, see 
      * {@link #setIssuer(byte []) setIssuer(byte [] issuerDN)}.<br />
      * <br />
-     * Uses {@link org.bouncycastle.asn1.DERInputStream DERInputStream},
+     * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
      * {@link org.bouncycastle.asn1.DERObject DERObject},
      * {@link org.bouncycastle.asn1.ASN1Sequence ASN1Sequence},
      * {@link org.bouncycastle.asn1.x509.X509Name X509Name}
@@ -338,7 +347,7 @@ public class X509CertSelector implements CertSelector
         else
         {
             ByteArrayInputStream inStream = new ByteArrayInputStream(subjectDN);
-            DERInputStream derInStream = new DERInputStream(inStream);
+            ASN1InputStream derInStream = new ASN1InputStream(inStream);
             DERObject obj = derInStream.readObject();
 
             if (obj instanceof ASN1Sequence)
@@ -1798,7 +1807,7 @@ public class X509CertSelector implements CertSelector
      * <br />
      * <b>TODO: implement output for currently unsupported options(name constraints)</b><br />
      * <br />
-     * Uses {@link org.bouncycastle.asn1.DERInputStream DERInputStream},
+     * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
      * {@link org.bouncycastle.asn1.DERObject DERObject},
      * {@link org.bouncycastle.asn1.x509.KeyPurposeId KeyPurposeId} and
      * {@link org.bouncycastle.asn1.util.ASN1Dump#_dumpAsString _dumpAsString}
@@ -1823,14 +1832,14 @@ public class X509CertSelector implements CertSelector
             if ( subjectKeyID != null )
             {
             ByteArrayInputStream inStream = new ByteArrayInputStream(subjectKeyID);
-            DERInputStream derInStream = new DERInputStream(inStream);
+            ASN1InputStream derInStream = new ASN1InputStream(inStream);
             DERObject derObject = derInStream.readObject();
             sb.append("  Subject Key Identifier: ").append(ASN1Dump.dumpAsString(derObject)).append('\n');
             }
             if ( authorityKeyID != null )
             {
             ByteArrayInputStream inStream = new ByteArrayInputStream(authorityKeyID);
-            DERInputStream derInStream = new DERInputStream(inStream);
+            ASN1InputStream derInStream = new ASN1InputStream(inStream);
             DERObject derObject = derInStream.readObject();
             sb.append("  Authority Key Identifier: ").append(ASN1Dump.dumpAsString(derObject)).append('\n');
             }
@@ -1863,7 +1872,7 @@ public class X509CertSelector implements CertSelector
             {
                 obj = (List)iter.next();
                 ByteArrayInputStream inStream = new ByteArrayInputStream((byte[])obj.get(1));
-                DERInputStream derInStream = new DERInputStream(inStream);
+                ASN1InputStream derInStream = new ASN1InputStream(inStream);
                 DERObject derObject = derInStream.readObject();
                 sb.append("  Type: ").append(obj.get(0)).append(" Data: ").append(ASN1Dump.dumpAsString(derObject)).append('\n');
             }
@@ -1882,7 +1891,7 @@ public class X509CertSelector implements CertSelector
             {
                 obj = (List)iter.next();
                 ByteArrayInputStream inStream = new ByteArrayInputStream((byte[])obj.get(1));
-                DERInputStream derInStream = new DERInputStream(inStream);
+                ASN1InputStream derInStream = new ASN1InputStream(inStream);
                 DERObject derObject = derInStream.readObject();
                 sb.append("  Type: ").append(obj.get(0)).append(" Data: ").append(ASN1Dump.dumpAsString(derObject)).append('\n');
             }
@@ -1900,7 +1909,7 @@ public class X509CertSelector implements CertSelector
      * <br />
      * <b>TODO: implement missing tests (name constraints and path to names)</b><br />
      * <br />
-     * Uses {@link org.bouncycastle.asn1.DERInputStream DERInputStream},
+     * Uses {@link org.bouncycastle.asn1.ASN1InputStream ASN1InputStream},
      * {@link org.bouncycastle.asn1.ASN1Sequence ASN1Sequence},
      * {@link org.bouncycastle.asn1.DERObjectIdentifier DERObjectIdentifier},
      * {@link org.bouncycastle.asn1.DERObject DERObject},
@@ -1953,7 +1962,7 @@ public class X509CertSelector implements CertSelector
             return false;
             try {
             ByteArrayInputStream inStream = new ByteArrayInputStream(data);
-            DERInputStream derInputStream = new DERInputStream(inStream);
+            ASN1InputStream derInputStream = new ASN1InputStream(inStream);
             byte[] testData = ((ASN1OctetString)derInputStream.readObject()).getOctets();
             if ( ! Arrays.equals( subjectKeyID, testData ) )
                 return false;
@@ -1968,7 +1977,7 @@ public class X509CertSelector implements CertSelector
             return false;
             try {
             ByteArrayInputStream inStream = new ByteArrayInputStream(data);
-            DERInputStream derInputStream = new DERInputStream(inStream);
+            ASN1InputStream derInputStream = new ASN1InputStream(inStream);
             byte[] testData = ((ASN1OctetString)derInputStream.readObject()).getOctets();
             if ( ! Arrays.equals( authorityKeyID, testData ) )
                 return false;
@@ -1990,9 +1999,9 @@ public class X509CertSelector implements CertSelector
             if ( data != null )
             {
                 ByteArrayInputStream inStream = new ByteArrayInputStream(data);
-                DERInputStream derInputStream = new DERInputStream(inStream);
+                ASN1InputStream derInputStream = new ASN1InputStream(inStream);
                 inStream = new ByteArrayInputStream(((ASN1OctetString)derInputStream.readObject()).getOctets());
-                derInputStream = new DERInputStream(inStream);
+                derInputStream = new ASN1InputStream(inStream);
                 //TODO fix this, Sequence contains tagged objects
                 ASN1Sequence derObject = (ASN1Sequence)derInputStream.readObject();
                 DERGeneralizedTime derDate = DERGeneralizedTime.getInstance( derObject.getObjectAt(0) );
@@ -2012,7 +2021,7 @@ public class X509CertSelector implements CertSelector
         {
             try {
             ByteArrayInputStream inStream = new ByteArrayInputStream(certX509.getPublicKey().getEncoded());
-            DERInputStream derInputStream = new DERInputStream(inStream);
+            ASN1InputStream derInputStream = new ASN1InputStream(inStream);
             SubjectPublicKeyInfo publicKeyInfo = new SubjectPublicKeyInfo((ASN1Sequence)derInputStream.readObject());
             AlgorithmIdentifier algInfo = publicKeyInfo.getAlgorithmId();
             if ( ! algInfo.getObjectId().equals(subjectKeyAlgID) )
@@ -2043,7 +2052,7 @@ public class X509CertSelector implements CertSelector
                 byte[] data = certX509.getExtensionValue( X509Extensions.ExtendedKeyUsage.getId() );
                 if ( data != null ) {
                 ByteArrayInputStream inStream = new ByteArrayInputStream(data);
-                DERInputStream derInputStream = new DERInputStream(inStream);
+                ASN1InputStream derInputStream = new ASN1InputStream(inStream);
                 ExtendedKeyUsage extendedKeyUsage = new ExtendedKeyUsage((ASN1Sequence)derInputStream.readObject());
                 tempIter = keyPurposeSet.iterator();
                 while( tempIter.hasNext() ) {
@@ -2070,9 +2079,9 @@ public class X509CertSelector implements CertSelector
                 if ( ! policyOID.isEmpty() )
                 {
                 ByteArrayInputStream inStream = new ByteArrayInputStream(data);
-                DERInputStream derInputStream = new DERInputStream(inStream);
+                ASN1InputStream derInputStream = new ASN1InputStream(inStream);
                 inStream = new ByteArrayInputStream(((ASN1OctetString)derInputStream.readObject()).getOctets());
-                derInputStream = new DERInputStream(inStream);
+                derInputStream = new ASN1InputStream(inStream);
                 Enumeration policySequence = ((ASN1Sequence)derInputStream.readObject()).getObjects();
                 ASN1Sequence policyObject;
                 boolean test = false;
@@ -2100,9 +2109,9 @@ public class X509CertSelector implements CertSelector
                         return false;
                     }
                     ByteArrayInputStream inStream = new ByteArrayInputStream(data);
-                    DERInputStream derInputStream = new DERInputStream(inStream);
+                    ASN1InputStream derInputStream = new ASN1InputStream(inStream);
                     inStream = new ByteArrayInputStream(((ASN1OctetString)derInputStream.readObject()).getOctets());
-                    derInputStream = new DERInputStream(inStream);
+                    derInputStream = new ASN1InputStream(inStream);
                     Enumeration altNamesSequence = ((ASN1Sequence)derInputStream.readObject()).getObjects();
                     ASN1TaggedObject altNameObject;
                     boolean test = false;
