@@ -32,19 +32,15 @@ import java.security.SignatureException;
 import java.security.cert.CRLException;
 import java.security.cert.X509CRL;
 import java.security.cert.X509CRLEntry;
-import java.text.SimpleDateFormat;
 import java.util.Date;
 import java.util.Iterator;
 import java.util.Set;
-import java.util.SimpleTimeZone;
 
 /**
  * class to produce an X.509 Version 2 CRL.
  */
 public class X509V2CRLGenerator
 {
-    private SimpleDateFormat            dateF = new SimpleDateFormat("yyMMddHHmmss");
-    private SimpleTimeZone              tz = new SimpleTimeZone(0, "Z");
     private V2TBSCertListGenerator      tbsGen;
     private DERObjectIdentifier         sigOID;
     private AlgorithmIdentifier         sigAlgId;
@@ -53,8 +49,6 @@ public class X509V2CRLGenerator
 
     public X509V2CRLGenerator()
     {
-        dateF.setTimeZone(tz);
-
         tbsGen = new V2TBSCertListGenerator();
         extGenerator = new X509ExtensionsGenerator();
     }
@@ -93,7 +87,7 @@ public class X509V2CRLGenerator
     /**
      * Reason being as indicated by CRLReason, i.e. CRLReason.keyCompromise
      * or 0 if CRLReason is not to be used
-     */
+     **/
     public void addCRLEntry(BigInteger userCertificate, Date revocationDate, int reason)
     {
         tbsGen.addCRLEntry(new DERInteger(userCertificate), new Time(revocationDate), reason);
@@ -103,12 +97,12 @@ public class X509V2CRLGenerator
      * Add a CRL entry with an Invalidity Date extension as well as a CRLReason extension.
      * Reason being as indicated by CRLReason, i.e. CRLReason.keyCompromise
      * or 0 if CRLReason is not to be used
-     */
+     **/
     public void addCRLEntry(BigInteger userCertificate, Date revocationDate, int reason, Date invalidityDate)
     {
         tbsGen.addCRLEntry(new DERInteger(userCertificate), new Time(revocationDate), reason, new DERGeneralizedTime(invalidityDate));
     }
-
+   
     /**
      * Add a CRL entry with extensions.
      **/
@@ -297,6 +291,10 @@ public class X509V2CRLGenerator
         {
             throw e;
         }
+        catch (NoSuchAlgorithmException e)
+        {
+            throw new SecurityException("exception: " + e);
+        }
         catch (GeneralSecurityException e)
         {
             throw new SecurityException("exception: " + e);
@@ -424,6 +422,7 @@ public class X509V2CRLGenerator
         ExtCRLException(String message, Throwable cause)
         {
             super(message);
+            this.cause = cause;
         }
 
         public Throwable getCause()
