@@ -1,53 +1,43 @@
 package org.bouncycastle.cms;
 
+import java.io.IOException;
+import java.security.AlgorithmParameterGenerator;
+import java.security.AlgorithmParameters;
+import java.security.InvalidAlgorithmParameterException;
+import java.security.InvalidKeyException;
+import java.security.KeyPair;
+import java.security.NoSuchAlgorithmException;
+import java.security.NoSuchProviderException;
+import java.security.PrivateKey;
+import java.security.Provider;
+import java.security.PublicKey;
+import java.security.SecureRandom;
+import java.security.cert.X509Certificate;
+import java.util.ArrayList;
+import java.util.Collection;
+import java.util.Collections;
+import java.util.List;
+
+import javax.crypto.SecretKey;
+import javax.crypto.spec.RC2ParameterSpec;
+import javax.crypto.spec.SecretKeySpec;
+
 import org.bouncycastle.asn1.ASN1Object;
 import org.bouncycastle.asn1.DEREncodable;
 import org.bouncycastle.asn1.DERNull;
 import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.DEROctetString;
 import org.bouncycastle.asn1.cms.KEKIdentifier;
-import org.bouncycastle.asn1.cms.OriginatorIdentifierOrKey;
-import org.bouncycastle.asn1.cms.OriginatorPublicKey;
 import org.bouncycastle.asn1.kisa.KISAObjectIdentifiers;
 import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
 import org.bouncycastle.asn1.ntt.NTTObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PBKDF2Params;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
-import org.bouncycastle.asn1.x509.SubjectPublicKeyInfo;
 import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
 
-import javax.crypto.KeyAgreement;
-import javax.crypto.SecretKey;
-import javax.crypto.spec.RC2ParameterSpec;
-import javax.crypto.spec.SecretKeySpec;
-import java.io.IOException;
-import java.security.AlgorithmParameterGenerator;
-import java.security.AlgorithmParameters;
-import java.security.InvalidAlgorithmParameterException;
-import java.security.InvalidKeyException;
-import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
-import java.security.PrivateKey;
-import java.security.PublicKey;
-import java.security.SecureRandom;
-import java.security.Provider;
-import java.security.cert.X509Certificate;
-import java.util.ArrayList;
-import java.util.List;
-
 /**
  * General class for generating a CMS enveloped-data message.
- *
- * A simple example of usage.
- *
- * <pre>
- *      CMSEnvelopedDataGenerator  fact = new CMSEnvelopedDataGenerator();
- *
- *      fact.addKeyTransRecipient(cert);
- *
- *      CMSEnvelopedData         data = fact.generate(content, algorithm, "BC");
- * </pre>
  */
 public class CMSEnvelopedGenerator
 {
@@ -73,9 +63,13 @@ public class CMSEnvelopedGenerator
     public static final String  SEED_WRAP       = KISAObjectIdentifiers.id_npki_app_cmsSeed_wrap.getId();
 
     public static final String  ECDH_SHA1KDF    = X9ObjectIdentifiers.dhSinglePass_stdDH_sha1kdf_scheme.getId();
-//    public static final String  ECMQV_SHA1KDF   = X9ObjectIdentifiers.mqvSinglePass_sha1kdf_scheme.getId();
+    public static final String  ECMQV_SHA1KDF   = X9ObjectIdentifiers.mqvSinglePass_sha1kdf_scheme.getId();
 
+    final List oldRecipientInfoGenerators = new ArrayList();
     final List recipientInfoGenerators = new ArrayList();
+
+    protected CMSAttributeTableGenerator unprotectedAttributeGenerator = null;
+
     final SecureRandom rand;
 
     /**
@@ -96,9 +90,15 @@ public class CMSEnvelopedGenerator
         this.rand = rand;
     }
 
+    public void setUnprotectedAttributeGenerator(CMSAttributeTableGenerator unprotectedAttributeGenerator)
+    {
+        this.unprotectedAttributeGenerator = unprotectedAttributeGenerator;
+    }
+
     /**
      * add a recipient.
      *
+     * @deprecated use the addRecipientGenerator and JceKeyTransRecipientInfoGenerator
      * @param cert recipient's public key certificate
      * @exception IllegalArgumentException if there is a problem with the certificate
      */
@@ -106,15 +106,16 @@ public class CMSEnvelopedGenerator
         X509Certificate cert)
         throws IllegalArgumentException
     {
-        KeyTransRecipientInfoGenerator ktrig = new KeyTransRecipientInfoGenerator();
+        KeyTransIntRecipientInfoGenerator ktrig = new KeyTransIntRecipientInfoGenerator();
         ktrig.setRecipientCert(cert);
 
-        recipientInfoGenerators.add(ktrig);
+        oldRecipientInfoGenerators.add(ktrig);
     }
 
     /**
      * add a recipient
      *
+     * @deprecated use the addRecipientGenerator and JceKeyTransRecipientInfoGenerator
      * @param key the public key used by the recipient
      * @param subKeyId the identifier for the recipient's public key
      * @exception IllegalArgumentException if there is a problem with the key
@@ -124,15 +125,17 @@ public class CMSEnvelopedGenerator
         byte[]      subKeyId)
         throws IllegalArgumentException
     {
-        KeyTransRecipientInfoGenerator ktrig = new KeyTransRecipientInfoGenerator();
+        KeyTransIntRecipientInfoGenerator ktrig = new KeyTransIntRecipientInfoGenerator();
         ktrig.setRecipientPublicKey(key);
         ktrig.setSubjectKeyIdentifier(new DEROctetString(subKeyId));
 
-        recipientInfoGenerators.add(ktrig);
+        oldRecipientInfoGenerators.add(ktrig);
     }
 
     /**
      * add a KEK recipient.
+     *
+     * @deprecated use the addRecipientGenerator and JceKEKRecipientInfoGenerator
      * @param key the secret key to use for wrapping
      * @param keyIdentifier the byte string that identifies the key
      */
@@ -140,29 +143,49 @@ public class CMSEnvelopedGenerator
         SecretKey   key,
         byte[]      keyIdentifier)
     {
-        KEKRecipientInfoGenerator kekrig = new KEKRecipientInfoGenerator();
-        kekrig.setKEKIdentifier(new KEKIdentifier(keyIdentifier, null, null));
-        kekrig.setWrapKey(key);
-
-        recipientInfoGenerators.add(kekrig);
+        addKEKRecipient(key, new KEKIdentifier(keyIdentifier, null, null));
     }
 
+    /**
+     * add a KEK recipient.
+     *
+     * @deprecated use the addRecipientGenerator and JceKEKRecipientInfoGenerator
+     * @param key the secret key to use for wrapping
+     * @param kekIdentifier a KEKIdentifier structure (identifies the key)
+     */
+    public void addKEKRecipient(
+        SecretKey       key,
+        KEKIdentifier   kekIdentifier)
+    {
+        KEKIntRecipientInfoGenerator kekrig = new KEKIntRecipientInfoGenerator();
+        kekrig.setKEKIdentifier(kekIdentifier);
+        kekrig.setKeyEncryptionKey(key);
+
+        oldRecipientInfoGenerators.add(kekrig);
+    }
+
+    /**
+     * @deprecated use addRecipientGenerator and JcePasswordRecipientInfoGenerator
+     * @param pbeKey PBE key
+     * @param kekAlgorithmOid key encryption algorithm to use.
+     */
     public void addPasswordRecipient(
         CMSPBEKey pbeKey,
         String    kekAlgorithmOid)
     {
         PBKDF2Params params = new PBKDF2Params(pbeKey.getSalt(), pbeKey.getIterationCount());
 
-        PasswordRecipientInfoGenerator prig = new PasswordRecipientInfoGenerator();
-        prig.setDerivationAlg(new AlgorithmIdentifier(PKCSObjectIdentifiers.id_PBKDF2, params));
-        prig.setWrapKey(new SecretKeySpec(pbeKey.getEncoded(kekAlgorithmOid), kekAlgorithmOid));
+        PasswordIntRecipientInfoGenerator prig = new PasswordIntRecipientInfoGenerator();
+        prig.setKeyDerivationAlgorithm(new AlgorithmIdentifier(PKCSObjectIdentifiers.id_PBKDF2, params));
+        prig.setKeyEncryptionKey(new SecretKeySpec(pbeKey.getEncoded(kekAlgorithmOid), kekAlgorithmOid));
 
-        recipientInfoGenerators.add(prig);
+        oldRecipientInfoGenerators.add(prig);
     }
 
     /**
      * Add a key agreement based recipient.
      *
+     * @deprecated use the addRecipientGenerator and JceKeyAgreeRecipientInfoGenerator
      * @param agreementAlgorithm key agreement algorithm to use.
      * @param senderPrivateKey private key to initialise sender side of agreement with.
      * @param senderPublicKey sender public key to include with message.
@@ -188,6 +211,7 @@ public class CMSEnvelopedGenerator
     /**
      * Add a key agreement based recipient.
      *
+     * @deprecated use the addRecipientGenerator and JceKeyAgreeRecipientInfoGenerator
      * @param agreementAlgorithm key agreement algorithm to use.
      * @param senderPrivateKey private key to initialise sender side of agreement with.
      * @param senderPublicKey sender public key to include with message.
@@ -206,35 +230,79 @@ public class CMSEnvelopedGenerator
         Provider         provider)
         throws NoSuchAlgorithmException, InvalidKeyException
     {
-        KeyAgreement agreement = KeyAgreement.getInstance(agreementAlgorithm, provider);
+        addKeyAgreementRecipients(agreementAlgorithm, senderPrivateKey, senderPublicKey,
+            Collections.singletonList(recipientCert), cekWrapAlgorithm, provider);
+    }
 
-        agreement.init(senderPrivateKey, rand);
+    /**
+     * Add multiple key agreement based recipients (sharing a single KeyAgreeRecipientInfo structure).
+     *
+     * @deprecated use the addRecipientGenerator and JceKeyAgreeRecipientInfoGenerator
+     * @param agreementAlgorithm key agreement algorithm to use.
+     * @param senderPrivateKey private key to initialise sender side of agreement with.
+     * @param senderPublicKey sender public key to include with message.
+     * @param recipientCerts recipients' public key certificates.
+     * @param cekWrapAlgorithm OID for key wrapping algorithm to use.
+     * @param provider provider to use for the agreement calculation.
+     * @exception NoSuchAlgorithmException if the algorithm requested cannot be found
+     * @exception InvalidKeyException if the keys are inappropriate for the algorithm specified
+     */
+    public void addKeyAgreementRecipients(
+        String           agreementAlgorithm,
+        PrivateKey       senderPrivateKey,
+        PublicKey        senderPublicKey,
+        Collection       recipientCerts,
+        String           cekWrapAlgorithm,
+        String           provider)
+        throws NoSuchProviderException, NoSuchAlgorithmException, InvalidKeyException
+    {
+        addKeyAgreementRecipients(agreementAlgorithm, senderPrivateKey, senderPublicKey, recipientCerts, cekWrapAlgorithm, CMSUtils.getProvider(provider));
+    }
 
-        agreement.doPhase(recipientCert.getPublicKey(), true);
+    /**
+     * Add multiple key agreement based recipients (sharing a single KeyAgreeRecipientInfo structure).
+     *
+     * @deprecated use the addRecipientGenerator and JceKeyAgreeRecipientInfoGenerator
+     * @param agreementAlgorithm key agreement algorithm to use.
+     * @param senderPrivateKey private key to initialise sender side of agreement with.
+     * @param senderPublicKey sender public key to include with message.
+     * @param recipientCerts recipients' public key certificates.
+     * @param cekWrapAlgorithm OID for key wrapping algorithm to use.
+     * @param provider provider to use for the agreement calculation.
+     * @exception NoSuchAlgorithmException if the algorithm requested cannot be found
+     * @exception InvalidKeyException if the keys are inappropriate for the algorithm specified
+     */
+    public void addKeyAgreementRecipients(
+        String           agreementAlgorithm,
+        PrivateKey       senderPrivateKey,
+        PublicKey        senderPublicKey,
+        Collection       recipientCerts,
+        String           cekWrapAlgorithm,
+        Provider         provider)
+        throws NoSuchAlgorithmException, InvalidKeyException
+    {
+        /* TODO
+         * "a recipient X.509 version 3 certificate that contains a key usage extension MUST
+         * assert the keyAgreement bit."
+         */
 
-        try
-        {
-            SubjectPublicKeyInfo oPubKeyInfo = SubjectPublicKeyInfo.getInstance(ASN1Object.fromByteArray(senderPublicKey.getEncoded()));
-            OriginatorIdentifierOrKey originator = new OriginatorIdentifierOrKey(
-                                                       new OriginatorPublicKey(
-                                                            new AlgorithmIdentifier(oPubKeyInfo.getAlgorithmId().getObjectId(), new DERNull()),
-                                                            oPubKeyInfo.getPublicKeyData().getBytes()));
+        KeyAgreeIntRecipientInfoGenerator karig = new KeyAgreeIntRecipientInfoGenerator();
+        karig.setKeyAgreementOID(new DERObjectIdentifier(agreementAlgorithm));
+        karig.setKeyEncryptionOID(new DERObjectIdentifier(cekWrapAlgorithm));
+        karig.setRecipientCerts(recipientCerts);
+        karig.setSenderKeyPair(new KeyPair(senderPublicKey, senderPrivateKey));
 
-            SecretKey wrapKey = agreement.generateSecret(cekWrapAlgorithm);
+        oldRecipientInfoGenerators.add(karig);
+    }
 
-            KeyAgreeRecipientInfoGenerator karig = new KeyAgreeRecipientInfoGenerator();
-            karig.setAlgorithmOID(new DERObjectIdentifier(agreementAlgorithm));
-            karig.setOriginator(originator);
-            karig.setRecipientCert(recipientCert);
-            karig.setWrapKey(wrapKey);
-            karig.setWrapAlgorithmOID(new DERObjectIdentifier(cekWrapAlgorithm));
-
-            recipientInfoGenerators.add(karig);
-        }
-        catch (IOException e)
-        {
-            throw new InvalidKeyException("cannot extract originator public key: " + e);
-        }
+    /**
+     * Add a generator to produce the recipient info required.
+     * 
+     * @param recipientGenerator a generator of a recipient info object.
+     */
+    public void addRecipientInfoGenerator(RecipientInfoGenerator recipientGenerator)
+    {
+        recipientInfoGenerators.add(recipientGenerator);
     }
 
     protected AlgorithmIdentifier getAlgorithmIdentifier(String encryptionOID, AlgorithmParameters params) throws IOException
@@ -279,13 +347,13 @@ public class CMSEnvelopedGenerator
 
             return pGen.generateParameters();
         }
+        catch (NoSuchProviderException e)
+        {
+            return null;
+        }
         catch (NoSuchAlgorithmException e)
         {
             return null;
         }
-        catch (NoSuchProviderException e)
-        {
-            throw new CMSException("exception generating the algorithm parameters", e);
-        }
     }
 }
