@@ -1,5 +1,24 @@
 package org.bouncycastle.cms;
 
+import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.security.InvalidAlgorithmParameterException;
+import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
+import java.security.NoSuchProviderException;
+import java.security.Provider;
+import java.security.Signature;
+import java.security.cert.CRLException;
+import org.bouncycastle.jce.cert.CertStore;
+import java.security.cert.CertificateException;
+import java.security.cert.CertificateFactory;
+import org.bouncycastle.jce.cert.CollectionCertStoreParameters;
+import java.util.ArrayList;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+
 import org.bouncycastle.asn1.ASN1Sequence;
 import org.bouncycastle.asn1.ASN1Set;
 import org.bouncycastle.asn1.ASN1TaggedObject;
@@ -7,8 +26,8 @@ import org.bouncycastle.asn1.DEREncodable;
 import org.bouncycastle.asn1.DERNull;
 import org.bouncycastle.asn1.DERObject;
 import org.bouncycastle.asn1.DERObjectIdentifier;
-import org.bouncycastle.asn1.eac.EACObjectIdentifiers;
 import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
+import org.bouncycastle.asn1.eac.EACObjectIdentifiers;
 import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
 import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
@@ -21,25 +40,6 @@ import org.bouncycastle.x509.X509CollectionStoreParameters;
 import org.bouncycastle.x509.X509Store;
 import org.bouncycastle.x509.X509V2AttributeCertificate;
 
-import java.io.ByteArrayInputStream;
-import java.io.IOException;
-import java.security.InvalidAlgorithmParameterException;
-import java.security.MessageDigest;
-import java.security.NoSuchAlgorithmException;
-import java.security.NoSuchProviderException;
-import java.security.Signature;
-import java.security.Provider;
-import java.security.cert.CRLException;
-import org.bouncycastle.jce.cert.CertStore;
-import java.security.cert.CertificateException;
-import java.security.cert.CertificateFactory;
-import org.bouncycastle.jce.cert.CollectionCertStoreParameters;
-import java.util.ArrayList;
-import java.util.Enumeration;
-import java.util.HashMap;
-import java.util.List;
-import java.util.Map;
-
 class CMSSignedHelper
 {
     static final CMSSignedHelper INSTANCE = new CMSSignedHelper();
@@ -98,6 +98,8 @@ class CMSSignedHelper
         encryptionAlgs.put(CryptoProObjectIdentifiers.gostR3410_2001.getId(), "ECGOST3410");
         encryptionAlgs.put("1.3.6.1.4.1.5849.1.6.2", "ECGOST3410");
         encryptionAlgs.put("1.3.6.1.4.1.5849.1.1.5", "GOST3410");
+        encryptionAlgs.put(CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_2001.getId(), "ECGOST3410");
+        encryptionAlgs.put(CryptoProObjectIdentifiers.gostR3411_94_with_gostR3410_94.getId(), "GOST3410");
 
         digestAlgs.put(PKCSObjectIdentifiers.md2.getId(), "MD2");
         digestAlgs.put(PKCSObjectIdentifiers.md4.getId(), "MD4");
@@ -173,8 +175,6 @@ class CMSSignedHelper
         Provider provider)
         throws NoSuchAlgorithmException
     {
-        try
-        {
         try
         {
             return createDigestInstance(algorithm, provider);
@@ -199,21 +199,23 @@ class CMSSignedHelper
             }
             throw e;
         }
-        }
-        catch (NoSuchProviderException e)
-        {
-            return getDigestInstance(algorithm, null); // try rolling back
-        }
     }
 
     private MessageDigest createDigestInstance(
         String algorithm,
         Provider provider)
-        throws NoSuchAlgorithmException, NoSuchProviderException
+        throws NoSuchAlgorithmException
     {
         if (provider != null)
         {
+            try
+            {
             return MessageDigest.getInstance(algorithm, provider.getName());
+            }
+            catch (NoSuchProviderException e)
+            {
+                throw new NoSuchAlgorithmException(e.toString());
+            }
         }
         else
         {
@@ -224,11 +226,18 @@ class CMSSignedHelper
     Signature getSignatureInstance(
         String algorithm, 
         Provider provider)
-        throws NoSuchAlgorithmException, NoSuchProviderException
+        throws NoSuchAlgorithmException
     {
         if (provider != null)
         {
+            try
+            {
             return Signature.getInstance(algorithm, provider.getName());
+            }
+            catch (NoSuchProviderException e)
+            {
+                throw new NoSuchAlgorithmException(e.toString());
+            }
         }
         else
         {
@@ -387,13 +396,13 @@ class CMSSignedHelper
                 cf = CertificateFactory.getInstance("X.509");
             }
         }
-        catch (CertificateException ex)
+        catch (NoSuchProviderException ex)
         {
             throw new CMSException("can't get certificate factory.", ex);
         }
-        catch (NoSuchProviderException ex)
+        catch (CertificateException ex)
         {
-            throw new CMSException("can't get provider for certificate factory.", ex);
+            throw new CMSException("can't get certificate factory.", ex);
         }
         Enumeration e = certSet.getObjects();
 
@@ -438,13 +447,13 @@ class CMSSignedHelper
                 cf = CertificateFactory.getInstance("X.509");
             }
         }
-        catch (CertificateException ex)
+        catch (NoSuchProviderException ex)
         {
             throw new CMSException("can't get certificate factory.", ex);
         }
-        catch (NoSuchProviderException ex)
+        catch (CertificateException ex)
         {
-            throw new CMSException("can't get certificate factory provider.", ex);
+            throw new CMSException("can't get certificate factory.", ex);
         }
         Enumeration e = certSet.getObjects();
 
@@ -477,4 +486,14 @@ class CMSSignedHelper
 
         return algId;
     }
+
+    void setSigningEncryptionAlgorithmMapping(DERObjectIdentifier oid, String algorithmName)
+    {
+        encryptionAlgs.put(oid.getId(), algorithmName);
+    }
+
+    void setSigningDigestAlgorithmMapping(DERObjectIdentifier oid, String algorithmName)
+    {
+        digestAlgs.put(oid.getId(), algorithmName);
+    }
 }
