@@ -3,6 +3,7 @@ package org.bouncycastle.cms;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
+import java.security.MessageDigest;
 import java.security.NoSuchProviderException;
 import java.security.Provider;
 import java.security.Security;
@@ -13,6 +14,7 @@ import java.security.cert.CertificateEncodingException;
 import java.security.cert.X509CRL;
 import java.security.cert.X509Certificate;
 import java.util.ArrayList;
+import java.util.Collection;
 import java.util.Iterator;
 import java.util.List;
 
@@ -24,11 +26,19 @@ import org.bouncycastle.asn1.BEROctetStringGenerator;
 import org.bouncycastle.asn1.BERSet;
 import org.bouncycastle.asn1.DEREncodable;
 import org.bouncycastle.asn1.DERSet;
+import org.bouncycastle.asn1.DERTaggedObject;
 import org.bouncycastle.asn1.cms.ContentInfo;
+import org.bouncycastle.asn1.cms.IssuerAndSerialNumber;
 import org.bouncycastle.asn1.x509.CertificateList;
 import org.bouncycastle.asn1.x509.TBSCertificateStructure;
 import org.bouncycastle.asn1.x509.X509CertificateStructure;
+import org.bouncycastle.cert.X509AttributeCertificateHolder;
+import org.bouncycastle.cert.X509CRLHolder;
+import org.bouncycastle.cert.X509CertificateHolder;
+import org.bouncycastle.util.Store;
 import org.bouncycastle.util.io.Streams;
+import org.bouncycastle.util.io.TeeInputStream;
+import org.bouncycastle.util.io.TeeOutputStream;
 
 class CMSUtils
 {
@@ -36,14 +46,7 @@ class CMSUtils
     
     static int getMaximumMemory()
     {
-        long maxMem = Integer.MAX_VALUE;
-        
-        if (maxMem > Integer.MAX_VALUE)
-        {
             return Integer.MAX_VALUE;
-        }
-        
-        return (int)maxMem;
     }
     
     static ContentInfo readContentInfo(
@@ -93,6 +96,50 @@ class CMSUtils
         }
     }
 
+    static List getCertificatesFromStore(Store certStore)
+        throws CMSException
+    {
+        List certs = new ArrayList();
+
+        try
+        {
+            for (Iterator it = certStore.getMatches(null).iterator(); it.hasNext();)
+            {
+                X509CertificateHolder c = (X509CertificateHolder)it.next();
+
+                certs.add(c.toASN1Structure());
+            }
+
+            return certs;
+        }
+        catch (ClassCastException e)
+        {
+            throw new CMSException("error processing certs", e);
+        }
+    }
+
+    static List getAttributeCertificatesFromStore(Store attrStore)
+        throws CMSException
+    {
+        List certs = new ArrayList();
+
+        try
+        {
+            for (Iterator it = attrStore.getMatches(null).iterator(); it.hasNext();)
+            {
+                X509AttributeCertificateHolder attrCert = (X509AttributeCertificateHolder)it.next();
+
+                certs.add(new DERTaggedObject(false, 2, attrCert.toASN1Structure()));
+            }
+
+            return certs;
+        }
+        catch (ClassCastException e)
+        {
+            throw new CMSException("error processing certs", e);
+        }
+    }
+
     static List getCRLsFromStore(CertStore certStore)
         throws CertStoreException, CMSException
     {
@@ -123,6 +170,28 @@ class CMSUtils
         }
     }
 
+    static List getCRLsFromStore(Store crlStore)
+        throws CMSException
+    {
+        List certs = new ArrayList();
+
+        try
+        {
+            for (Iterator it = crlStore.getMatches(null).iterator(); it.hasNext();)
+            {
+                X509CRLHolder c = (X509CRLHolder)it.next();
+
+                certs.add(c.toASN1Structure());
+            }
+
+            return certs;
+        }
+        catch (ClassCastException e)
+        {
+            throw new CMSException("error processing certs", e);
+        }
+    }
+
     static ASN1Set createBerSetFromList(List derObjects)
     {
         ASN1EncodableVector v = new ASN1EncodableVector();
@@ -161,19 +230,26 @@ class CMSUtils
     }
 
     static TBSCertificateStructure getTBSCertificateStructure(
-        X509Certificate cert) throws CertificateEncodingException
+        X509Certificate cert)
     {
         try
         {
-            return TBSCertificateStructure.getInstance(ASN1Object
-                .fromByteArray(cert.getTBSCertificate()));
+            return TBSCertificateStructure.getInstance(
+                ASN1Object.fromByteArray(cert.getTBSCertificate()));
         }
-        catch (IOException e)
+        catch (Exception e)
         {
-            throw new CertificateEncodingException(e.toString());
+            throw new IllegalArgumentException(
+                "can't extract TBS structure from this cert");
         }
     }
 
+    static IssuerAndSerialNumber getIssuerAndSerialNumber(X509Certificate cert)
+    {
+        TBSCertificateStructure tbsCert = getTBSCertificateStructure(cert);
+        return new IssuerAndSerialNumber(tbsCert.getIssuer(), tbsCert.getSerialNumber().getValue());
+    }
+
     private static ContentInfo readContentInfo(
         ASN1InputStream in)
         throws CMSException
@@ -228,4 +304,53 @@ class CMSUtils
 
         return null; 
     }
+
+    static InputStream attachDigestsToInputStream(Collection digests, InputStream s)
+    {
+        InputStream result = s;
+        Iterator it = digests.iterator();
+        while (it.hasNext())
+        {
+            MessageDigest digest = (MessageDigest)it.next();
+            result = new TeeInputStream(result, new DigOutputStream(digest));
+        }
+        return result;
+    }
+
+    static OutputStream attachDigestsToOutputStream(Collection digests, OutputStream s)
+    {
+        OutputStream result = s;
+        Iterator it = digests.iterator();
+        while (it.hasNext())
+        {
+            MessageDigest digest = (MessageDigest)it.next();
+            result = getSafeTeeOutputStream(result, new DigOutputStream(digest));
+        }
+        return result;
+    }
+
+    static OutputStream attachSignersToOutputStream(Collection signers, OutputStream s)
+    {
+        OutputStream result = s;
+        Iterator it = signers.iterator();
+        while (it.hasNext())
+        {
+            SignerInfoGenerator signerGen = (SignerInfoGenerator)it.next();
+            result = getSafeTeeOutputStream(result, signerGen.getCalculatingOutputStream());
+        }
+        return result;
+    }
+
+    static OutputStream getSafeOutputStream(OutputStream s)
+    {
+        return s == null ? new NullOutputStream() : s;
+    }
+
+    static OutputStream getSafeTeeOutputStream(OutputStream s1,
+            OutputStream s2)
+    {
+        return s1 == null ? getSafeOutputStream(s2)
+                : s2 == null ? getSafeOutputStream(s1) : new TeeOutputStream(
+                        s1, s2);
+    }
 }
