@@ -1,32 +1,12 @@
 package org.bouncycastle.mail.smime;
 
-import org.bouncycastle.asn1.cms.AttributeTable;
-import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
-import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
-import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
-import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
-import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
-import org.bouncycastle.cms.CMSException;
-import org.bouncycastle.cms.CMSSignedDataStreamGenerator;
-import org.bouncycastle.cms.SignerInformationStore;
-import org.bouncycastle.mail.smime.util.CRLFOutputStream;
-import org.bouncycastle.x509.X509Store;
-
-import javax.activation.CommandMap;
-import javax.activation.MailcapCommandMap;
-import javax.mail.MessagingException;
-import javax.mail.Multipart;
-import javax.mail.internet.ContentType;
-import javax.mail.internet.MimeBodyPart;
-import javax.mail.internet.MimeMessage;
-import javax.mail.internet.MimeMultipart;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.security.InvalidKeyException;
 import java.security.NoSuchAlgorithmException;
 import java.security.NoSuchProviderException;
 import java.security.PrivateKey;
+import java.security.Provider;
 import org.bouncycastle.jce.cert.CertStore;
 import org.bouncycastle.jce.cert.CertStoreException;
 import java.security.cert.X509Certificate;
@@ -39,6 +19,31 @@ import java.util.List;
 import java.util.Map;
 import java.util.Set;
 
+import javax.activation.CommandMap;
+import javax.activation.MailcapCommandMap;
+import javax.mail.MessagingException;
+import javax.mail.Multipart;
+import javax.mail.internet.ContentType;
+import javax.mail.internet.MimeBodyPart;
+import javax.mail.internet.MimeMessage;
+import javax.mail.internet.MimeMultipart;
+
+import org.bouncycastle.asn1.cms.AttributeTable;
+import org.bouncycastle.asn1.cryptopro.CryptoProObjectIdentifiers;
+import org.bouncycastle.asn1.nist.NISTObjectIdentifiers;
+import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
+import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.teletrust.TeleTrusTObjectIdentifiers;
+import org.bouncycastle.asn1.x9.X9ObjectIdentifiers;
+import org.bouncycastle.cms.CMSException;
+import org.bouncycastle.cms.CMSSignedDataStreamGenerator;
+import org.bouncycastle.cms.SignerInfoGenerator;
+import org.bouncycastle.cms.SignerInformation;
+import org.bouncycastle.cms.SignerInformationStore;
+import org.bouncycastle.mail.smime.util.CRLFOutputStream;
+import org.bouncycastle.util.Store;
+import org.bouncycastle.x509.X509Store;
+
 /**
  * general class for generating a pkcs7-signature message.
  * <p>
@@ -88,6 +93,10 @@ public class SMIMESignedGenerator
     private final String        _defaultContentTransferEncoding;
 
     private List                _certStores = new ArrayList();
+    private List                certStores = new ArrayList();
+    private List                crlStores = new ArrayList();
+    private List                attrCertStores = new ArrayList();
+    private List                signerInfoGens = new ArrayList();
     private List                _signers = new ArrayList();
     private List                _oldSigners = new ArrayList();
     private List                _attributeCerts = new ArrayList();
@@ -95,15 +104,20 @@ public class SMIMESignedGenerator
     
     static
     {
-        MailcapCommandMap mc = (MailcapCommandMap)CommandMap.getDefaultCommandMap();
+        CommandMap.setDefaultCommandMap(addCommands(CommandMap.getDefaultCommandMap()));
+    }
+
+    private static MailcapCommandMap addCommands(CommandMap cm)
+    {
+        MailcapCommandMap mc = (MailcapCommandMap)cm;
 
         mc.addMailcap("application/pkcs7-signature;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.pkcs7_signature");
         mc.addMailcap("application/pkcs7-mime;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.pkcs7_mime");
         mc.addMailcap("application/x-pkcs7-signature;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.x_pkcs7_signature");
         mc.addMailcap("application/x-pkcs7-mime;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.x_pkcs7_mime");
         mc.addMailcap("multipart/signed;; x-java-content-handler=org.bouncycastle.mail.smime.handlers.multipart_signed");
-        
-        CommandMap.setDefaultCommandMap(mc);
+
+        return mc;
     }
 
     /**
@@ -133,6 +147,7 @@ public class SMIMESignedGenerator
      * @param cert the public key certificate associated with the signer's key.
      * @param digestOID object ID of the digest algorithm to use.
      * @exception IllegalArgumentException any of the arguments are inappropriate
+     * @deprecated use addSignerInfoGenerator()
      */
     public void addSigner(
         PrivateKey      key,
@@ -143,6 +158,27 @@ public class SMIMESignedGenerator
         _signers.add(new Signer(key, cert, digestOID, null, null));
     }
 
+    /**
+     * add a signer - no attributes other than the default ones will be
+     * provided here.
+     *
+     * @param key key to use to generate the signature
+     * @param cert the public key certificate associated with the signer's key.
+     * @param encryptionOID object ID of the digest ecnryption algorithm to use.
+     * @param digestOID object ID of the digest algorithm to use.
+     * @exception IllegalArgumentException any of the arguments are inappropriate
+     * @deprecated use addSignerInfoGenerator()
+     */
+    public void addSigner(
+        PrivateKey      key,
+        X509Certificate cert,
+        String          encryptionOID,
+        String          digestOID)
+        throws IllegalArgumentException
+    {
+        _signers.add(new Signer(key, cert, encryptionOID, digestOID, null, null));
+    }
+
     /**
      * Add a signer with extra signed/unsigned attributes or overrides
      * for the standard attributes. For example this method can be used to
@@ -154,6 +190,7 @@ public class SMIMESignedGenerator
      * @param signedAttr signed attributes to be included in the signature.
      * @param unsignedAttr unsigned attribitues to be included.
      * @exception IllegalArgumentException any of the arguments are inappropriate
+     * @deprecated use addSignerInfoGenerator()
      */
     public void addSigner(
         PrivateKey      key,
@@ -166,6 +203,33 @@ public class SMIMESignedGenerator
         _signers.add(new Signer(key, cert, digestOID, signedAttr, unsignedAttr));
     }
 
+    /**
+     * Add a signer with extra signed/unsigned attributes or overrides
+     * for the standard attributes and a digest encryption algorithm. For
+     * example this method can be used to explictly set default attributes
+     * such as the signing time.
+     *
+     * @param key key to use to generate the signature
+     * @param cert the public key certificate associated with the signer's key.
+     * @param encryptionOID the digest encryption algorithm OID.
+     * @param digestOID object ID of the digest algorithm to use.
+     * @param signedAttr signed attributes to be included in the signature.
+     * @param unsignedAttr unsigned attribitues to be included.
+     * @exception IllegalArgumentException any of the arguments are inappropriate
+     * @deprecated use addSignerInfoGenerator()
+     */
+    public void addSigner(
+        PrivateKey      key,
+        X509Certificate cert,
+        String          encryptionOID,
+        String          digestOID,
+        AttributeTable  signedAttr,
+        AttributeTable  unsignedAttr)
+        throws IllegalArgumentException
+    {
+        _signers.add(new Signer(key, cert, encryptionOID, digestOID, signedAttr, unsignedAttr));
+    }
+
     /**
      * Add a store of precalculated signers to the generator.
      *
@@ -182,6 +246,11 @@ public class SMIMESignedGenerator
         }
     }
 
+    public void addSignerInfoGenerator(SignerInfoGenerator sigInfoGen)
+    {
+        signerInfoGens.add(sigInfoGen);
+    }
+
     /**
      * add the certificates and CRLs contained in the given CertStore
      * to the pool that will be included in the encoded signature block.
@@ -190,6 +259,7 @@ public class SMIMESignedGenerator
      * methods.
      * </p>
      * @param certStore CertStore containing the certificates and CRLs to be added.
+     * @deprecated use addCertificates(Store) and addCRLs(Store)
      */
     public void addCertificatesAndCRLs(
         CertStore               certStore)
@@ -198,12 +268,31 @@ public class SMIMESignedGenerator
         _certStores.add(certStore);
     }
 
+    public void addCertificates(
+        Store certStore)
+    {
+        certStores.add(certStore);
+    }
+
+    public void addCRLs(
+        Store crlStore)
+    {
+        crlStores.add(crlStore);
+    }
+
+    public void addAttributeCertificates(
+        Store certStore)
+    {
+        attrCertStores.add(certStore);
+    }
+
     /**
      * Add the attribute certificates contained in the passed in store to the
      * generator.
      *
      * @param store a store of Version 2 attribute certificates
      * @throws CMSException if an error occurse processing the store.
+     * @deprecated use addAttributeCertificates(Store)
      */
     public void addAttributeCertificates(
         X509Store store)
@@ -226,33 +315,47 @@ public class SMIMESignedGenerator
         
         while (it.hasNext())
         {
-            Signer       signer = (Signer)it.next();
-            
-            if (signer.getDigestOID().equals(DIGEST_SHA1))
+            Object       signer = it.next();
+            String       digestOID;
+
+            if (signer instanceof Signer)
+            {
+                digestOID = ((Signer)signer).getDigestOID();
+            }
+            else if (signer instanceof SignerInformation)
+            {
+                digestOID = ((SignerInformation)signer).getDigestAlgOID();
+            }
+            else
+            {
+                digestOID = ((SignerInfoGenerator)signer).getDigestAlgorithm().getAlgorithm().getId();
+            }
+
+            if (digestOID.equals(DIGEST_SHA1))
             {
                 micAlgs.add("sha1");
             }
-            else if (signer.getDigestOID().equals(DIGEST_MD5))
+            else if (digestOID.equals(DIGEST_MD5))
             {
                 micAlgs.add("md5");
             }
-            else if (signer.getDigestOID().equals(DIGEST_SHA224))
+            else if (digestOID.equals(DIGEST_SHA224))
             {
                 micAlgs.add("sha224");
             }
-            else if (signer.getDigestOID().equals(DIGEST_SHA256))
+            else if (digestOID.equals(DIGEST_SHA256))
             {
                 micAlgs.add("sha256");
             }
-            else if (signer.getDigestOID().equals(DIGEST_SHA384))
+            else if (digestOID.equals(DIGEST_SHA384))
             {
                 micAlgs.add("sha384");
             }
-            else if (signer.getDigestOID().equals(DIGEST_SHA512))
+            else if (digestOID.equals(DIGEST_SHA512))
             {
                 micAlgs.add("sha512");
             }
-            else if (signer.getDigestOID().equals(DIGEST_GOST3411))
+            else if (digestOID.equals(DIGEST_GOST3411))
             {
                 micAlgs.add("gostr3411-94");
             }
@@ -303,8 +406,8 @@ public class SMIMESignedGenerator
      */
     private MimeMultipart make(
         MimeBodyPart    content,
-        String          sigProvider)
-        throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
+        Provider        sigProvider)
+        throws NoSuchAlgorithmException, SMIMEException
     {
         try
         {
@@ -321,9 +424,56 @@ public class SMIMESignedGenerator
             //
             StringBuffer        header = new StringBuffer(
                     "signed; protocol=\"application/pkcs7-signature\"");
-                    
-            addHashHeader(header, _signers);
-            
+
+            List allSigners = new ArrayList(_signers);
+
+            allSigners.addAll(_oldSigners);
+
+            allSigners.addAll(signerInfoGens);
+
+            addHashHeader(header, allSigners);
+
+            MimeMultipart   mm = new MimeMultipart(header.toString());
+
+            mm.addBodyPart(content);
+            mm.addBodyPart(sig);
+
+            return mm;
+        }
+        catch (MessagingException e)
+        {
+            throw new SMIMEException("exception putting multi-part together.", e);
+        }
+    }
+
+    private MimeMultipart make(
+        MimeBodyPart    content)
+    throws NoSuchAlgorithmException, SMIMEException
+    {
+        try
+        {
+            MimeBodyPart sig = new MimeBodyPart();
+
+            sig.setContent(new ContentSigner(content, false), DETACHED_SIGNATURE_TYPE);
+            sig.addHeader("Content-Type", DETACHED_SIGNATURE_TYPE);
+            sig.addHeader("Content-Disposition", "attachment; filename=\"smime.p7s\"");
+            sig.addHeader("Content-Description", "S/MIME Cryptographic Signature");
+            sig.addHeader("Content-Transfer-Encoding", encoding);
+
+            //
+            // build the multipart header
+            //
+            StringBuffer        header = new StringBuffer(
+                    "signed; protocol=\"application/pkcs7-signature\"");
+
+            List allSigners = new ArrayList(_signers);
+
+            allSigners.addAll(_oldSigners);
+
+            allSigners.addAll(signerInfoGens);
+
+            addHashHeader(header, allSigners);
+
             MimeMultipart   mm = new MimeMultipart(header.toString());
 
             mm.addBodyPart(content);
@@ -342,8 +492,8 @@ public class SMIMESignedGenerator
      */
     private MimeBodyPart makeEncapsulated(
         MimeBodyPart    content,
-        String          sigProvider)
-        throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
+        Provider        sigProvider)
+        throws NoSuchAlgorithmException, SMIMEException
     {
         try
         {
@@ -363,6 +513,31 @@ public class SMIMESignedGenerator
         }
     }
 
+    /*
+     * at this point we expect our body part to be well defined - generate with data in the signature
+     */
+    private MimeBodyPart makeEncapsulated(
+        MimeBodyPart    content)
+        throws NoSuchAlgorithmException, SMIMEException
+    {
+        try
+        {
+            MimeBodyPart sig = new MimeBodyPart();
+
+            sig.setContent(new ContentSigner(content, true), ENCAPSULATED_SIGNED_CONTENT_TYPE);
+            sig.addHeader("Content-Type", ENCAPSULATED_SIGNED_CONTENT_TYPE);
+            sig.addHeader("Content-Disposition", "attachment; filename=\"smime.p7m\"");
+            sig.addHeader("Content-Description", "S/MIME Cryptographic Signed Data");
+            sig.addHeader("Content-Transfer-Encoding", encoding);
+
+            return sig;
+        }
+        catch (MessagingException e)
+        {
+            throw new SMIMEException("exception putting body part together.", e);
+        }
+    }
+
     /**
      * Return a map of oids and byte arrays representing the digests calculated on the content during
      * the last generate.
@@ -383,11 +558,29 @@ public class SMIMESignedGenerator
      * @throws NoSuchAlgorithmException if the required algorithms for the signature cannot be found.
      * @throws NoSuchProviderException if no provider can be found.
      * @throws SMIMEException if an exception occurs in processing the signature.
+     * @deprecated use generate(MimeBodyPart)
      */
     public MimeMultipart generate(
         MimeBodyPart    content,
         String          sigProvider)
         throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
+    {
+        return make(makeContentBodyPart(content), SMIMEUtil.getProvider(sigProvider));
+    }
+
+    /**
+     * generate a signed object that contains an SMIME Signed Multipart
+     * object using the given provider.
+     * @param content the MimeBodyPart to be signed.
+     * @param sigProvider the provider to be used for the signature.
+     * @return a Multipart containing the content and signature.
+     * @throws NoSuchAlgorithmException if the required algorithms for the signature cannot be found.
+     * @throws SMIMEException if an exception occurs in processing the signature.
+     */
+    public MimeMultipart generate(
+        MimeBodyPart    content,
+        Provider        sigProvider)
+        throws NoSuchAlgorithmException, SMIMEException
     {
         return make(makeContentBodyPart(content), sigProvider);
     }
@@ -404,6 +597,22 @@ public class SMIMESignedGenerator
         MimeMessage     message,
         String          sigProvider)
         throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
+    {
+        return generate(message, SMIMEUtil.getProvider(sigProvider));
+    }
+
+    /**
+     * generate a signed object that contains an SMIME Signed Multipart
+     * object using the given provider from the given MimeMessage
+     *
+     * @throws NoSuchAlgorithmException if the required algorithms for the signature cannot be found.
+     * @throws NoSuchProviderException if no provider can be found.
+     * @throws SMIMEException if an exception occurs in processing the signature.
+     */
+    public MimeMultipart generate(
+        MimeMessage     message,
+        Provider        sigProvider)
+        throws NoSuchAlgorithmException, SMIMEException
     {
         try
         {
@@ -417,17 +626,55 @@ public class SMIMESignedGenerator
         return make(makeContentBodyPart(message), sigProvider);
     }
 
+    public MimeMultipart generate(
+        MimeBodyPart    content)
+        throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
+    {
+        return make(makeContentBodyPart(content));
+    }
+
+    /**
+     * generate a signed message with encapsulated content
+     * <p>
+     * Note: doing this is strongly <b>not</b> recommended as it means a
+     * recipient of the message will have to be able to read the signature to read the
+     * message.
+     */
+    public MimeBodyPart generateEncapsulated(
+        MimeBodyPart    content)
+        throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
+    {
+        return makeEncapsulated(makeContentBodyPart(content));
+    }
+
     /**
      * generate a signed message with encapsulated content
      * <p>
      * Note: doing this is strongly <b>not</b> recommended as it means a
      * recipient of the message will have to be able to read the signature to read the 
      * message.
+     * @deprecated use generateEncapsulated(content)
      */
     public MimeBodyPart generateEncapsulated(
         MimeBodyPart    content,
         String          sigProvider)
         throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
+    {
+        return makeEncapsulated(makeContentBodyPart(content), SMIMEUtil.getProvider(sigProvider));
+    }
+
+    /**
+     * generate a signed message with encapsulated content
+     * <p>
+     * Note: doing this is strongly <b>not</b> recommended as it means a
+     * recipient of the message will have to be able to read the signature to read the
+     * message.
+     * @deprecated use generateEncapsulated(content)
+     */
+    public MimeBodyPart generateEncapsulated(
+        MimeBodyPart    content,
+        Provider        sigProvider)
+        throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
     {
         return makeEncapsulated(makeContentBodyPart(content), sigProvider);
     }
@@ -437,13 +684,31 @@ public class SMIMESignedGenerator
      * object using the given provider from the given MimeMessage.
      * <p>
      * Note: doing this is strongly <b>not</b> recommended as it means a
-     * recipient of the message will have to be able to read the signature to read the 
+     * recipient of the message will have to be able to read the signature to read the
      * message.
+     * @deprecated use generateEncapsulated(content)
      */
     public MimeBodyPart generateEncapsulated(
         MimeMessage     message,
         String          sigProvider)
         throws NoSuchAlgorithmException, NoSuchProviderException, SMIMEException
+    {
+        return generateEncapsulated(message, SMIMEUtil.getProvider(sigProvider));
+    }
+
+    /**
+     * generate a signed object that contains an SMIME Signed Multipart
+     * object using the given provider from the given MimeMessage.
+     * <p>
+     * Note: doing this is strongly <b>not</b> recommended as it means a
+     * recipient of the message will have to be able to read the signature to read the 
+     * message.
+     * @deprecated use generateEncapsulated(content)
+     */
+    public MimeBodyPart generateEncapsulated(
+        MimeMessage     message,
+        Provider        sigProvider)
+        throws NoSuchAlgorithmException, SMIMEException
     {
         try
         {
@@ -456,7 +721,20 @@ public class SMIMESignedGenerator
 
         return makeEncapsulated(makeContentBodyPart(message), sigProvider);
     }
-    
+
+    /**
+     * Creates a certificate management message which is like a signed message with no content
+     * or signers but that still carries certificates and CRLs.
+     *
+     * @return a MimeBodyPart containing the certs and CRLs.
+     */
+    public MimeBodyPart generateCertificateManagement(
+       String provider)
+       throws SMIMEException, NoSuchProviderException
+    {
+        return generateCertificateManagement(SMIMEUtil.getProvider(provider));
+    }
+
     /**
      * Creates a certificate management message which is like a signed message with no content
      * or signers but that still carries certificates and CRLs.
@@ -464,8 +742,8 @@ public class SMIMESignedGenerator
      * @return a MimeBodyPart containing the certs and CRLs.
      */
     public MimeBodyPart generateCertificateManagement(
-       String provider) 
-       throws SMIMEException, NoSuchProviderException
+       Provider provider)
+       throws SMIMEException
     {
         try
         {
@@ -489,6 +767,7 @@ public class SMIMESignedGenerator
     {
         final PrivateKey      key;
         final X509Certificate cert;
+        final String          encryptionOID;
         final String          digestOID;
         final AttributeTable  signedAttr;
         final AttributeTable  unsignedAttr;
@@ -499,9 +778,21 @@ public class SMIMESignedGenerator
             String          digestOID,
             AttributeTable  signedAttr,
             AttributeTable  unsignedAttr)
+        {
+            this(key, cert, null, digestOID, signedAttr, unsignedAttr);
+        }
+
+        Signer(
+            PrivateKey      key,
+            X509Certificate cert,
+            String          encryptionOID,
+            String          digestOID,
+            AttributeTable  signedAttr,
+            AttributeTable  unsignedAttr)
         {
             this.key = key;
             this.cert = cert;
+            this.encryptionOID = encryptionOID;
             this.digestOID = digestOID;
             this.signedAttr = signedAttr;
             this.unsignedAttr = unsignedAttr;
@@ -512,6 +803,11 @@ public class SMIMESignedGenerator
             return cert;
         }
 
+        public String getEncryptionOID()
+        {
+            return encryptionOID;
+        }
+
         public String getDigestOID()
         {
             return digestOID;
@@ -536,20 +832,32 @@ public class SMIMESignedGenerator
     private class ContentSigner
         implements SMIMEStreamingProcessor
     {
-        private final MimeBodyPart _content;
-        private final boolean      _encapsulate;
-        private final String       _provider;
+        private final MimeBodyPart content;
+        private final boolean encapsulate;
+        private final Provider provider;
+        private final boolean  noProvider;
 
         ContentSigner(
             MimeBodyPart content,
             boolean      encapsulate,
-            String       provider)
+            Provider     provider)
         {
-            _content = content;
-            _encapsulate = encapsulate;
-            _provider = provider;
+            this.content = content;
+            this.encapsulate = encapsulate;
+            this.provider = provider;
+            this.noProvider = false;
         }
-        
+
+        ContentSigner(
+            MimeBodyPart content,
+            boolean      encapsulate)
+        {
+            this.content = content;
+            this.encapsulate = encapsulate;
+            this.provider = null;
+            this.noProvider = true;
+        }
+
         protected CMSSignedDataStreamGenerator getGenerator()
             throws CMSException, CertStoreException, InvalidKeyException, NoSuchAlgorithmException, NoSuchProviderException
         {
@@ -560,6 +868,21 @@ public class SMIMESignedGenerator
                 gen.addCertificatesAndCRLs((CertStore)it.next());
             }
 
+            for (Iterator it = certStores.iterator(); it.hasNext();)
+            {
+                gen.addCertificates((Store)it.next());
+            }
+
+            for (Iterator it = crlStores.iterator(); it.hasNext();)
+            {
+                gen.addCRLs((Store)it.next());
+            }
+
+            for (Iterator it = attrCertStores.iterator(); it.hasNext();)
+            {
+                gen.addAttributeCertificates((Store)it.next());
+            }
+
             for (Iterator it = _attributeCerts.iterator(); it.hasNext();)
             {
                 gen.addAttributeCertificates((X509Store)it.next());
@@ -568,8 +891,20 @@ public class SMIMESignedGenerator
             for (Iterator it = _signers.iterator(); it.hasNext();)
             {
                 Signer signer = (Signer)it.next();
-                
-                gen.addSigner(signer.getKey(), signer.getCert(), signer.getDigestOID(), signer.getSignedAttr(), signer.getUnsignedAttr(), _provider);
+
+                if (signer.getEncryptionOID() != null)
+                {
+                    gen.addSigner(signer.getKey(), signer.getCert(), signer.getEncryptionOID(), signer.getDigestOID(), signer.getSignedAttr(), signer.getUnsignedAttr(), provider);
+                }
+                else
+                {
+                    gen.addSigner(signer.getKey(), signer.getCert(), signer.getDigestOID(), signer.getSignedAttr(), signer.getUnsignedAttr(), provider);
+                }
+            }
+
+            for (Iterator it = signerInfoGens.iterator(); it.hasNext();)
+            {
+                gen.addSignerInfoGenerator((SignerInfoGenerator)it.next());
             }
 
             gen.addSigners(new SignerInformationStore(_oldSigners));
@@ -627,17 +962,19 @@ public class SMIMESignedGenerator
             {
                 CMSSignedDataStreamGenerator gen = getGenerator();
                 
-                OutputStream signingStream = gen.open(out, _encapsulate);
+                OutputStream signingStream = gen.open(out, encapsulate);
                 
-                if (_content != null)
+                if (content != null)
                 {
-                    if (!_encapsulate)
+                    if (!encapsulate)
                     {
-                        writeBodyPart(signingStream, _content);
+                        writeBodyPart(signingStream, content);
                     }
                     else
                     {
-                        _content.writeTo(signingStream);
+                        content.getDataHandler().setCommandMap(addCommands(CommandMap.getDefaultCommandMap()));
+
+                        content.writeTo(signingStream);
                     }
                 }
                 
