@@ -1,26 +1,30 @@
 package org.bouncycastle.cms;
 
-import java.security.SecureRandom;
-import java.security.Provider;
+import java.io.IOException;
 import java.security.AlgorithmParameterGenerator;
 import java.security.AlgorithmParameters;
 import java.security.GeneralSecurityException;
 import java.security.NoSuchAlgorithmException;
+import java.security.Provider;
+import java.security.SecureRandom;
 import java.security.spec.AlgorithmParameterSpec;
 import java.security.spec.InvalidParameterSpecException;
-import java.io.OutputStream;
-import java.io.IOException;
+import java.util.HashMap;
+import java.util.Map;
 
-import javax.crypto.Mac;
 import javax.crypto.SecretKey;
-import javax.crypto.spec.RC2ParameterSpec;
 import javax.crypto.spec.IvParameterSpec;
+import javax.crypto.spec.RC2ParameterSpec;
 
+import org.bouncycastle.asn1.DERObjectIdentifier;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 
 public class CMSAuthenticatedGenerator
     extends CMSEnvelopedGenerator
 {
+    protected CMSAttributeTableGenerator authGen;
+    protected CMSAttributeTableGenerator unauthGen;
+
     /**
      * base constructor
      */
@@ -74,48 +78,22 @@ public class CMSAuthenticatedGenerator
         }
     }
 
-    protected class MacOutputStream
-        extends OutputStream
+    public void setAuthenticatedAttributeGenerator(CMSAttributeTableGenerator authGen)
     {
-        private final OutputStream out;
-        private Mac mac;
+        this.authGen = authGen;
+    }
 
-        MacOutputStream(OutputStream out, Mac mac)
-        {
-            this.out = out;
-            this.mac = mac;
-        }
+    public void setUnauthenticatedAttributeGenerator(CMSAttributeTableGenerator unauthGen)
+    {
+        this.unauthGen = unauthGen;
+    }
 
-        public void write(byte[] buf)
-            throws IOException
-        {
-            mac.update(buf, 0, buf.length);
-            out.write(buf, 0, buf.length);
-        }
-
-        public void write(byte[] buf, int off, int len)
-            throws IOException
-        {
-            mac.update(buf, off, len);
-            out.write(buf, off, len);
-        }
-
-        public void write(int i)
-            throws IOException
-        {
-            mac.update((byte)i);
-            out.write(i);
-        }
-
-        public void close()
-            throws IOException
-        {
-            out.close();
-        }
-        
-        public byte[] getMac()
-        {
-            return mac.doFinal();
-        }
+    protected Map getBaseParameters(DERObjectIdentifier contentType, AlgorithmIdentifier digAlgId, byte[] hash)
+    {
+        Map param = new HashMap();
+        param.put(CMSAttributeTableGenerator.CONTENT_TYPE, contentType);
+        param.put(CMSAttributeTableGenerator.DIGEST_ALGORITHM_IDENTIFIER, digAlgId);
+        param.put(CMSAttributeTableGenerator.DIGEST,  hash.clone());
+        return param;
     }
 }
