@@ -1,29 +1,40 @@
 package org.bouncycastle.cms;
 
+import java.io.IOException;
+import java.io.InputStream;
+import java.security.AlgorithmParameters;
+import java.security.NoSuchProviderException;
+import java.security.Provider;
+
 import org.bouncycastle.asn1.ASN1Set;
 import org.bouncycastle.asn1.DEREncodable;
 import org.bouncycastle.asn1.cms.AttributeTable;
 import org.bouncycastle.asn1.cms.ContentInfo;
 import org.bouncycastle.asn1.cms.EncryptedContentInfo;
 import org.bouncycastle.asn1.cms.EnvelopedData;
-import org.bouncycastle.asn1.cms.KEKRecipientInfo;
-import org.bouncycastle.asn1.cms.KeyAgreeRecipientInfo;
-import org.bouncycastle.asn1.cms.KeyTransRecipientInfo;
-import org.bouncycastle.asn1.cms.PasswordRecipientInfo;
-import org.bouncycastle.asn1.cms.RecipientInfo;
 import org.bouncycastle.asn1.x509.AlgorithmIdentifier;
 
-import java.io.ByteArrayInputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.security.AlgorithmParameters;
-import java.security.NoSuchProviderException;
-import java.security.Provider;
-import java.util.ArrayList;
-import java.util.List;
-
 /**
  * containing class for an CMS Enveloped Data object
+ * <p>
+ * Example of use - assuming the first recipient matches the private key we have.
+ * <pre>
+ *      CMSEnvelopedData     ed = new CMSEnvelopedData(inputStream);
+ *
+ *      RecipientInformationStore  recipients = ed.getRecipientInfos();
+ *
+ *      Collection  c = recipients.getRecipients();
+ *      Iterator    it = c.iterator();
+ *
+ *      if (it.hasNext())
+ *      {
+ *          RecipientInformation   recipient = (RecipientInformation)it.next();
+ *
+ *          byte[] recData = recipient.getContent(new JceKeyTransEnvelopedRecipient(privateKey).setProvider("BC"));
+ *
+ *          processData(recData);
+ *      }
+ *  </pre>
  */
 public class CMSEnvelopedData
 {
@@ -49,55 +60,31 @@ public class CMSEnvelopedData
 
     public CMSEnvelopedData(
         ContentInfo contentInfo)
-        throws CMSException
     {
         this.contentInfo = contentInfo;
 
         EnvelopedData  envData = EnvelopedData.getInstance(contentInfo.getContent());
 
+        //
+        // read the recipients
+        //
+        ASN1Set recipientInfos = envData.getRecipientInfos();
+
         //
         // read the encrypted content info
         //
         EncryptedContentInfo encInfo = envData.getEncryptedContentInfo();
-
         this.encAlg = encInfo.getContentEncryptionAlgorithm();
+        CMSReadable readable = new CMSProcessableByteArray(encInfo.getEncryptedContent().getOctets());
+        CMSSecureReadable secureReadable = new CMSEnvelopedHelper.CMSEnvelopedSecureReadable(
+            this.encAlg, readable);
 
         //
-        // load the RecipientInfoStore
+        // build the RecipientInformationStore
         //
-        ASN1Set     s = envData.getRecipientInfos();
-        List        infos = new ArrayList();
-        byte[]      contentOctets = encInfo.getEncryptedContent().getOctets();
+        this.recipientInfoStore = CMSEnvelopedHelper.buildRecipientInformationStore(
+            recipientInfos, this.encAlg, secureReadable);
 
-        for (int i = 0; i != s.size(); i++)
-        {
-            RecipientInfo   info = RecipientInfo.getInstance(s.getObjectAt(i));
-            InputStream     contentStream = new ByteArrayInputStream(contentOctets);
-            Object          type = info.getInfo();
-
-            if (type instanceof KeyTransRecipientInfo)
-            {
-                infos.add(new KeyTransRecipientInformation(
-                    (KeyTransRecipientInfo)type, encAlg, contentStream));
-            }
-            else if (type instanceof KEKRecipientInfo)
-            {
-                infos.add(new KEKRecipientInformation(
-                    (KEKRecipientInfo)type, encAlg, contentStream));
-            }
-            else if (type instanceof KeyAgreeRecipientInfo)
-            {
-                infos.add(new KeyAgreeRecipientInformation(
-                    (KeyAgreeRecipientInfo)type, encAlg, contentStream));
-            }
-            else if (type instanceof PasswordRecipientInfo)
-            {
-                infos.add(new PasswordRecipientInformation(
-                    (PasswordRecipientInfo)type, encAlg, contentStream));
-            }
-        }
-
-        this.recipientInfoStore = new RecipientInformationStore(infos);
         this.unprotectedAttributes = envData.getUnprotectedAttrs();
     }
 
