@@ -1,6 +1,6 @@
 package org.bouncycastle.cms;
 
-import java.io.InputStream;
+import java.io.IOException;
 import java.security.InvalidKeyException;
 import java.security.Key;
 import java.security.NoSuchAlgorithmException;
@@ -23,28 +23,19 @@ public class KEKRecipientInformation
 {
     private KEKRecipientInfo      info;
 
-    public KEKRecipientInformation(
+    KEKRecipientInformation(
         KEKRecipientInfo        info,
-        AlgorithmIdentifier     encAlg,
-        InputStream             data)
+        AlgorithmIdentifier     messageAlgorithm,
+        CMSSecureReadable       secureReadable,
+        AuthAttributesProvider  additionalData)
     {
-        this(info, encAlg, null, data);
-    }
+        super(info.getKeyEncryptionAlgorithm(), messageAlgorithm, secureReadable, additionalData);
 
-    public KEKRecipientInformation(
-        KEKRecipientInfo        info,
-        AlgorithmIdentifier     encAlg,
-        AlgorithmIdentifier     macAlg,
-        InputStream             data)
-    {
-        super(encAlg, macAlg, AlgorithmIdentifier.getInstance(info.getKeyEncryptionAlgorithm()), data);
-        
         this.info = info;
-        this.rid = new RecipientId();
-        
-        KEKIdentifier       kekId = info.getKekid();
 
-        rid.setKeyIdentifier(kekId.getKeyIdentifier().getOctets());
+        KEKIdentifier kekId = info.getKekid();
+
+        this.rid = new KEKRecipientId(kekId.getKeyIdentifier().getOctets());
     }
 
     /**
@@ -68,20 +59,11 @@ public class KEKRecipientInformation
     {
         try
         {
-            byte[]              encryptedKey = info.getEncryptedKey().getOctets();
-            Cipher              keyCipher = Cipher.getInstance(keyEncAlg.getObjectId().getId(), prov);
-
+            Cipher keyCipher = CMSEnvelopedHelper.INSTANCE.createSymmetricCipher(
+                keyEncAlg.getObjectId().getId(), prov);
             keyCipher.init(Cipher.UNWRAP_MODE, key);
-
-            AlgorithmIdentifier aid = encAlg;
-            if (aid == null)
-            {
-                aid = macAlg;
-            }
-            
-            String              alg = aid.getObjectId().getId();
-            Key                 sKey = keyCipher.unwrap(
-                                        encryptedKey, alg, Cipher.SECRET_KEY);
+            Key sKey = keyCipher.unwrap(info.getEncryptedKey().getOctets(), getContentAlgorithmName(),
+                Cipher.SECRET_KEY);
 
             return getContentFromSessionKey(sKey, prov);
         }
@@ -98,4 +80,10 @@ public class KEKRecipientInformation
             throw new CMSException("required padding not supported.", e);
         }
     }
+
+    protected RecipientOperator getRecipientOperator(Recipient recipient)
+        throws CMSException, IOException
+    {
+        return ((KEKRecipient)recipient).getRecipientOperator(keyEncAlg, messageAlgorithm, info.getEncryptedKey().getOctets());
+    }
 }
