@@ -7,7 +7,16 @@ import java.io.DataOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.OutputStream;
-import java.security.*;
+import java.security.Key;
+import java.security.KeyFactory;
+import java.security.KeyStoreException;
+import java.security.KeyStoreSpi;
+import java.security.NoSuchAlgorithmException;
+import java.security.NoSuchProviderException;
+import java.security.PrivateKey;
+import java.security.PublicKey;
+import java.security.SecureRandom;
+import java.security.UnrecoverableKeyException;
 import java.security.cert.Certificate;
 import java.security.cert.CertificateEncodingException;
 import java.security.cert.CertificateException;
@@ -27,6 +36,7 @@ import javax.crypto.spec.PBEKeySpec;
 import javax.crypto.spec.PBEParameterSpec;
 import javax.crypto.spec.SecretKeySpec;
 
+import org.bouncycastle.crypto.CipherParameters;
 import org.bouncycastle.crypto.Digest;
 import org.bouncycastle.crypto.PBEParametersGenerator;
 import org.bouncycastle.crypto.digests.SHA1Digest;
@@ -37,6 +47,8 @@ import org.bouncycastle.crypto.io.MacInputStream;
 import org.bouncycastle.crypto.io.MacOutputStream;
 import org.bouncycastle.crypto.macs.HMac;
 import org.bouncycastle.jce.interfaces.BCKeyStore;
+import org.bouncycastle.util.Arrays;
+import org.bouncycastle.util.io.Streams;
 
 public class JDKKeyStore
     extends KeyStoreSpi
@@ -94,17 +106,6 @@ public class JDKKeyStore
             this.certChain = null;
         }
 
-        StoreEntry(
-            String          alias,
-            Key             obj,
-            Certificate[]   certChain)
-        {
-            this.type = KEY;
-            this.alias = alias;
-            this.obj = obj;
-            this.certChain = certChain;
-        }
-
         StoreEntry(
             String          alias,
             byte[]          obj,
@@ -355,7 +356,7 @@ public class JDKKeyStore
 
         try
         {
-            CertificateFactory cFact = CertificateFactory.getInstance(type, "BC");
+            CertificateFactory cFact = CertificateFactory.getInstance(type, BouncyCastleProvider.PROVIDER_NAME);
             ByteArrayInputStream bIn = new ByteArrayInputStream(cEnc);
 
             return cFact.generateCertificate(bIn);
@@ -430,11 +431,11 @@ public class JDKKeyStore
             switch (keyType)
             {
             case KEY_PRIVATE:
-                return KeyFactory.getInstance(algorithm, "BC").generatePrivate(spec);
+                return KeyFactory.getInstance(algorithm, BouncyCastleProvider.PROVIDER_NAME).generatePrivate(spec);
             case KEY_PUBLIC:
-                return KeyFactory.getInstance(algorithm, "BC").generatePublic(spec);
+                return KeyFactory.getInstance(algorithm, BouncyCastleProvider.PROVIDER_NAME).generatePublic(spec);
             case KEY_SECRET:
-                return SecretKeyFactory.getInstance(algorithm, "BC").generateSecret(spec);
+                return SecretKeyFactory.getInstance(algorithm, BouncyCastleProvider.PROVIDER_NAME).generateSecret(spec);
             default:
                 throw new IOException("Key type " + keyType + " not recognised!");
             }
@@ -456,10 +457,10 @@ public class JDKKeyStore
         try
         {
             PBEKeySpec          pbeSpec = new PBEKeySpec(password);
-            SecretKeyFactory    keyFact = SecretKeyFactory.getInstance(algorithm, "BC");
+            SecretKeyFactory    keyFact = SecretKeyFactory.getInstance(algorithm, BouncyCastleProvider.PROVIDER_NAME);
             PBEParameterSpec    defParams = new PBEParameterSpec(salt, iterationCount);
 
-            Cipher cipher = Cipher.getInstance(algorithm, "BC");
+            Cipher cipher = Cipher.getInstance(algorithm, BouncyCastleProvider.PROVIDER_NAME);
 
             cipher.init(mode, keyFact.generateSecret(pbeSpec), defParams);
 
@@ -633,7 +634,7 @@ public class JDKKeyStore
 
         if (entry != null && entry.getType() != CERTIFICATE)
         {
-            throw new KeyStoreException("key store already has an entry with alias " + alias);
+            throw new KeyStoreException("key store already has a key entry with alias " + alias);
         }
 
         table.put(alias, new StoreEntry(alias, cert));
@@ -645,13 +646,6 @@ public class JDKKeyStore
         Certificate[] chain) 
         throws KeyStoreException
     {
-        StoreEntry  entry = (StoreEntry)table.get(alias);
-
-        if (entry != null)
-        {
-            throw new KeyStoreException("key store already has an entry with alias " + alias);
-        }
-
         table.put(alias, new StoreEntry(alias, key, chain));
     }
 
@@ -667,13 +661,6 @@ public class JDKKeyStore
             throw new KeyStoreException("no certificate chain for private key");
         }
 
-        StoreEntry  entry = (StoreEntry)table.get(alias);
-
-        if (entry != null && entry.getType() == CERTIFICATE)
-        {
-            throw new KeyStoreException("key store already has an entry with alias " + alias);
-        }
-
         try
         {
             table.put(alias, new StoreEntry(alias, key, password, chain));
@@ -689,26 +676,6 @@ public class JDKKeyStore
         return table.size();
     }
 
-    protected boolean isSameAs(
-        byte[]  one,
-        byte[]  two)
-    {
-        if (one.length != two.length)
-        {
-            return false;
-        }
-
-        for (int i = 0; i != one.length; i++)
-        {
-            if (one[i] != two[i])
-            {
-                return false;
-            }
-        }
-
-        return true;
-    }
-
     protected void loadStore(
         InputStream in)
         throws IOException
@@ -840,39 +807,47 @@ public class JDKKeyStore
 
         int         iterationCount = dIn.readInt();
 
-        HMac                    hMac = new HMac(new SHA1Digest());
-        MacInputStream          mIn = new MacInputStream(dIn, hMac);
-        PBEParametersGenerator  pbeGen = new PKCS12ParametersGenerator(new SHA1Digest());
-        byte[]                  passKey = PBEParametersGenerator.PKCS12PasswordToBytes(password);
-
-        pbeGen.init(passKey, salt, iterationCount);
-
-        hMac.init(pbeGen.generateDerivedMacParameters(hMac.getMacSize()));
-
-        for (int i = 0; i != passKey.length; i++)
-        {
-            passKey[i] = 0;
-        }
-
-        loadStore(mIn);
-
-        byte[]  mac = new byte[hMac.getMacSize()];
-        byte[]  oldMac = new byte[hMac.getMacSize()];
-
-        hMac.doFinal(mac, 0);
-
-        for (int i = 0; i != oldMac.length; i++)
-        {
-            oldMac[i] = (byte)dIn.read();
-        }
-
         //
         // we only do an integrity check if the password is provided.
         //
-        if ((password != null && password.length != 0) && !isSameAs(mac, oldMac))
+        HMac hMac = new HMac(new SHA1Digest());
+        if (password != null && password.length != 0)
         {
-            table.clear();
-            throw new IOException("KeyStore integrity check failed.");
+            byte[] passKey = PBEParametersGenerator.PKCS12PasswordToBytes(password);
+
+            PBEParametersGenerator pbeGen = new PKCS12ParametersGenerator(new SHA1Digest());
+            pbeGen.init(passKey, salt, iterationCount);
+            CipherParameters macParams = pbeGen.generateDerivedMacParameters(hMac.getMacSize());
+            Arrays.fill(passKey, (byte)0);
+
+            hMac.init(macParams);
+            MacInputStream mIn = new MacInputStream(dIn, hMac);
+
+            loadStore(mIn);
+
+            // Finalise our mac calculation
+            byte[] mac = new byte[hMac.getMacSize()];
+            hMac.doFinal(mac, 0);
+
+            // TODO Should this actually be reading the remainder of the stream?
+            // Read the original mac from the stream
+            byte[] oldMac = new byte[hMac.getMacSize()];
+            dIn.readFully(oldMac);
+
+            if (!Arrays.constantTimeAreEqual(mac, oldMac))
+            {
+                table.clear();
+                throw new IOException("KeyStore integrity check failed.");
+            }
+        }
+        else
+        {
+            loadStore(dIn);
+
+            // TODO Should this actually be reading the remainder of the stream?
+            // Parse the original mac from the stream too
+            byte[] oldMac = new byte[hMac.getMacSize()];
+            dIn.readFully(oldMac);
         }
     }
 
@@ -941,7 +916,6 @@ public class JDKKeyStore
                 return;
             }
     
-            Cipher              cipher;
             DataInputStream     dIn = new DataInputStream(stream);
             int                 version = dIn.readInt();
     
@@ -969,34 +943,34 @@ public class JDKKeyStore
                 throw new IOException("Key store corrupted.");
             }
     
+            String cipherAlg;
             if (version == 0)
             {
-                cipher = this.makePBECipher("Old" + STORE_CIPHER, Cipher.DECRYPT_MODE, password, salt, iterationCount);
+                cipherAlg = "Old" + STORE_CIPHER;
             }
             else
             {
-                cipher = this.makePBECipher(STORE_CIPHER, Cipher.DECRYPT_MODE, password, salt, iterationCount);
+                cipherAlg = STORE_CIPHER;
             }
-    
-            CipherInputStream  cIn = new CipherInputStream(dIn, cipher);
-    
-            DigestInputStream  dgIn = new DigestInputStream(cIn, new SHA1Digest());
+
+            Cipher cipher = this.makePBECipher(cipherAlg, Cipher.DECRYPT_MODE, password, salt, iterationCount);
+            CipherInputStream cIn = new CipherInputStream(dIn, cipher);
+
+            Digest dig = new SHA1Digest();
+            DigestInputStream  dgIn = new DigestInputStream(cIn, dig);
     
             this.loadStore(dgIn);
-    
-            Digest  dig = dgIn.getDigest();
-            int     digSize = dig.getDigestSize();
-            byte[]  hash = new byte[digSize];
-            byte[]  oldHash = new byte[digSize];
-    
+
+            // Finalise our digest calculation
+            byte[] hash = new byte[dig.getDigestSize()];
             dig.doFinal(hash, 0);
-    
-            for (int i = 0; i != digSize; i++)
-            {
-                oldHash[i] = (byte)cIn.read();
-            }
-    
-            if (!this.isSameAs(hash, oldHash))
+
+            // TODO Should this actually be reading the remainder of the stream?
+            // Read the original digest from the stream
+            byte[] oldHash = new byte[dig.getDigestSize()];
+            Streams.readFully(cIn, oldHash);
+
+            if (!Arrays.constantTimeAreEqual(hash, oldHash))
             {
                 table.clear();
                 throw new IOException("KeyStore integrity check failed.");
