@@ -1,13 +1,14 @@
 package org.bouncycastle.openpgp;
 
-import org.bouncycastle.bcpg.BCPGOutputStream;
-import org.bouncycastle.bcpg.PacketTags;
-
 import java.io.File;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.util.Date;
 
+import org.bouncycastle.bcpg.BCPGOutputStream;
+import org.bouncycastle.bcpg.PacketTags;
+import org.bouncycastle.util.Strings;
+
 /**
  * Class for producing literal data packets.
  */
@@ -15,6 +16,7 @@ public class PGPLiteralDataGenerator implements StreamGenerator
 {    
     public static final char    BINARY = PGPLiteralData.BINARY;
     public static final char    TEXT = PGPLiteralData.TEXT;
+    public static final char    UTF8 = PGPLiteralData.UTF8;
     
     /**
      * The special name indicating a "for your eyes only" packet.
@@ -54,11 +56,14 @@ public class PGPLiteralDataGenerator implements StreamGenerator
         throws IOException
     {
         out.write(format);
-        out.write((byte)name.length());
 
-        for (int i = 0; i != name.length(); i++)
+        byte[] encName = Strings.toUTF8ByteArray(name);
+
+        out.write((byte)encName.length);
+
+        for (int i = 0; i != encName.length; i++)
         {
-            out.write(name.charAt(i));
+            out.write(encName[i]);
         }
 
         long    modDate = modificationTime / 1000;
