@@ -1,5 +1,18 @@
 package org.bouncycastle.openpgp.examples;
 
+import java.io.BufferedInputStream;
+import java.io.BufferedOutputStream;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.OutputStream;
+import java.security.NoSuchProviderException;
+import java.security.SecureRandom;
+import java.security.Security;
+import java.util.Iterator;
+
 import org.bouncycastle.bcpg.ArmoredOutputStream;
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.openpgp.PGPCompressedData;
@@ -14,24 +27,9 @@ import org.bouncycastle.openpgp.PGPOnePassSignatureList;
 import org.bouncycastle.openpgp.PGPPrivateKey;
 import org.bouncycastle.openpgp.PGPPublicKey;
 import org.bouncycastle.openpgp.PGPPublicKeyEncryptedData;
-import org.bouncycastle.openpgp.PGPPublicKeyRing;
-import org.bouncycastle.openpgp.PGPPublicKeyRingCollection;
-import org.bouncycastle.openpgp.PGPSecretKey;
 import org.bouncycastle.openpgp.PGPSecretKeyRingCollection;
 import org.bouncycastle.openpgp.PGPUtil;
-
-import java.io.BufferedInputStream;
-import java.io.BufferedOutputStream;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.security.NoSuchProviderException;
-import java.security.SecureRandom;
-import java.security.Security;
-import java.util.Iterator;
+import org.bouncycastle.util.io.Streams;
 
 /**
  * A simple utility class that encrypts/decrypts public key based
@@ -56,83 +54,18 @@ import java.util.Iterator;
  */
 public class KeyBasedLargeFileProcessor
 {
-    /**
-     * A simple routine that opens a key ring file and loads the first available key suitable for
-     * encryption.
-     * 
-     * @param in
-     * @return
-     * @throws IOException
-     * @throws PGPException
-     */
-    private static PGPPublicKey readPublicKey(
-        InputStream    in)
-        throws IOException, PGPException
+    private static void decryptFile(
+        String inputFileName,
+        String keyFileName,
+        char[] passwd,
+        String defaultFileName)
+        throws IOException, NoSuchProviderException
     {
-        in = PGPUtil.getDecoderStream(in);
-        
-        PGPPublicKeyRingCollection        pgpPub = new PGPPublicKeyRingCollection(in);
-
-        //
-        // we just loop through the collection till we find a key suitable for encryption, in the real
-        // world you would probably want to be a bit smarter about this.
-        //
-        PGPPublicKey    key = null;
-        
-        //
-        // iterate through the key rings.
-        //
-        Iterator rIt = pgpPub.getKeyRings();
-        
-        while (key == null && rIt.hasNext())
-        {
-            PGPPublicKeyRing    kRing = (PGPPublicKeyRing)rIt.next();    
-            Iterator            kIt = kRing.getPublicKeys();
-            
-            while (key == null && kIt.hasNext())
-            {
-                PGPPublicKey    k = (PGPPublicKey)kIt.next();
-                
-                if (k.isEncryptionKey())
-                {
-                    key = k;
-                }
-            }
-        }
-        
-        if (key == null)
-        {
-            throw new IllegalArgumentException("Can't find encryption key in key ring.");
-        }
-        
-        return key;
-    }
-    
-    /**
-     * Search a secret key ring collection for a secret key corresponding to
-     * keyID if it exists.
-     * 
-     * @param pgpSec a secret key ring collection.
-     * @param keyID keyID we want.
-     * @param pass passphrase to decrypt secret key with.
-     * @return
-     * @throws PGPException
-     * @throws NoSuchProviderException
-     */
-    private static PGPPrivateKey findSecretKey(
-        PGPSecretKeyRingCollection  pgpSec,
-        long                        keyID,
-        char[]                      pass)
-        throws PGPException, NoSuchProviderException
-    {    
-        PGPSecretKey pgpSecKey = pgpSec.getSecretKey(keyID);
-        
-        if (pgpSecKey == null)
-        {
-            return null;
-        }
-        
-        return pgpSecKey.extractPrivateKey(pass, "BC");
+        InputStream in = new BufferedInputStream(new FileInputStream(inputFileName));
+        InputStream keyIn = new BufferedInputStream(new FileInputStream(keyFileName));
+        decryptFile(in, keyIn, passwd, defaultFileName);
+        keyIn.close();
+        in.close();
     }
     
     /**
@@ -143,7 +76,7 @@ public class KeyBasedLargeFileProcessor
         InputStream keyIn,
         char[]      passwd,
         String      defaultFileName)
-        throws Exception
+        throws IOException, NoSuchProviderException
     {    
         in = PGPUtil.getDecoderStream(in);
         
@@ -178,7 +111,7 @@ public class KeyBasedLargeFileProcessor
             {
                 pbe = (PGPPublicKeyEncryptedData)it.next();
                 
-                sKey = findSecretKey(pgpSec, pbe.getKeyID(), passwd);
+                sKey = PGPExampleUtil.findSecretKey(pgpSec, pbe.getKeyID(), passwd);
             }
             
             if (sKey == null)
@@ -199,25 +132,20 @@ public class KeyBasedLargeFileProcessor
             
             if (message instanceof PGPLiteralData)
             {
-                PGPLiteralData       ld = (PGPLiteralData)message;
+                PGPLiteralData ld = (PGPLiteralData)message;
 
-                String               outFileName = ld.getFileName();
+                String outFileName = ld.getFileName();
                 if (outFileName.length() == 0)
                 {
                     outFileName = defaultFileName;
                 }
-                FileOutputStream     fOut = new FileOutputStream(outFileName);
-                BufferedOutputStream bOut = new BufferedOutputStream(fOut);
-                
-                InputStream    unc = ld.getInputStream();
-                int    ch;
-                
-                while ((ch = unc.read()) >= 0)
-                {
-                    bOut.write(ch);
-                }
 
-                bOut.close();
+                InputStream unc = ld.getInputStream();
+                OutputStream fOut =  new BufferedOutputStream(new FileOutputStream(outFileName));
+
+                Streams.pipeAll(unc, fOut);
+
+                fOut.close();
             }
             else if (message instanceof PGPOnePassSignatureList)
             {
@@ -254,6 +182,20 @@ public class KeyBasedLargeFileProcessor
         }
     }
 
+    private static void encryptFile(
+        String          outputFileName,
+        String          inputFileName,
+        String          encKeyFileName,
+        boolean         armor,
+        boolean         withIntegrityCheck)
+        throws IOException, NoSuchProviderException, PGPException
+    {
+        OutputStream out = new BufferedOutputStream(new FileOutputStream(outputFileName));
+        PGPPublicKey encKey = PGPExampleUtil.readPublicKey(encKeyFileName);
+        encryptFile(out, inputFileName, encKey, armor, withIntegrityCheck);
+        out.close();
+    }
+
     private static void encryptFile(
         OutputStream    out,
         String          fileName,
@@ -284,7 +226,10 @@ public class KeyBasedLargeFileProcessor
             
             cOut.close();
 
-            out.close();
+            if (armor)
+            {
+                out.close();
+            }
         }
         catch (PGPException e)
         {
@@ -312,28 +257,20 @@ public class KeyBasedLargeFileProcessor
         {
             if (args[1].equals("-a") || args[1].equals("-ai") || args[1].equals("-ia"))
             {
-                FileInputStream     keyIn = new FileInputStream(args[3]);
-                FileOutputStream    out = new FileOutputStream(args[2] + ".asc");
-                encryptFile(out, args[2], readPublicKey(keyIn), true, (args[1].indexOf('i') > 0));
+                encryptFile(args[2] + ".asc", args[2], args[3], true, (args[1].indexOf('i') > 0));
             }
             else if (args[1].equals("-i"))
             {
-                FileInputStream     keyIn = new FileInputStream(args[3]);
-                FileOutputStream    out = new FileOutputStream(args[2] + ".bpg");
-                encryptFile(out, args[2], readPublicKey(keyIn), false, true);
+                encryptFile(args[2] + ".bpg", args[2], args[3], false, true);
             }
             else
             {
-                FileInputStream     keyIn = new FileInputStream(args[2]);
-                FileOutputStream    out = new FileOutputStream(args[1] + ".bpg");
-                encryptFile(out, args[1], readPublicKey(keyIn), false, false);
+                encryptFile(args[1] + ".bpg", args[1], args[2], false, false);
             }
         }
         else if (args[0].equals("-d"))
         {
-            FileInputStream    in = new FileInputStream(args[1]);
-            FileInputStream    keyIn = new FileInputStream(args[2]);
-            decryptFile(in, keyIn, args[3].toCharArray(), new File(args[1]).getName() + ".out");
+            decryptFile(args[1], args[2], args[3].toCharArray(), new File(args[1]).getName() + ".out");
         }
         else
         {
