@@ -1,17 +1,17 @@
 package org.bouncycastle.tsp;
 
 import java.io.ByteArrayInputStream;
-import java.io.InputStream;
 import java.io.IOException;
-import java.security.MessageDigest;
+import java.io.InputStream;
 
-import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
-import org.bouncycastle.asn1.tsp.TimeStampResp;
+import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.cmp.PKIFailureInfo;
 import org.bouncycastle.asn1.cmp.PKIFreeText;
 import org.bouncycastle.asn1.cmp.PKIStatus;
-import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.cms.Attribute;
+import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.asn1.tsp.TimeStampResp;
+import org.bouncycastle.util.Arrays;
 
 /**
  * Base class for an RFC 3161 Time Stamp Response object.
@@ -142,7 +142,7 @@ public class TimeStampResponse
                 throw new TSPValidationException("time stamp token found in failed request.");
             }
             
-            if (!MessageDigest.isEqual(request.getMessageImprintDigest(), tstInfo.getMessageImprintDigest()))
+            if (!Arrays.constantTimeAreEqual(request.getMessageImprintDigest(), tstInfo.getMessageImprintDigest()))
             {
                 throw new TSPValidationException("response for different message imprint digest.");
             }
