@@ -761,7 +761,9 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
         catch (CertPathReviewerException cpre)
         {
             addError(cpre.getErrorMessage());
-        } catch (Throwable t) {
+        }
+        catch (Throwable t)
+        {
             ErrorBundle msg = new ErrorBundle(RESOURCE_NAME,
                     "CertPathReviewer.unknown",
                     new Object[] {new UntrustedInput(t.getMessage()), new UntrustedInput(t)});
@@ -1555,14 +1557,14 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                                 switch (constraint.getTagNo())
                                 {
                                 case 0:
-                                    tmpInt = DERInteger.getInstance(constraint).getValue().intValue();
+                                    tmpInt = DERInteger.getInstance(constraint, false).getValue().intValue();
                                     if (tmpInt < explicitPolicy)
                                     {
                                         explicitPolicy = tmpInt;
                                     }
                                     break;
                                 case 1:
-                                    tmpInt = DERInteger.getInstance(constraint).getValue().intValue();
+                                    tmpInt = DERInteger.getInstance(constraint, false).getValue().intValue();
                                     if (tmpInt < policyMapping)
                                     {
                                         policyMapping = tmpInt;
@@ -1635,7 +1637,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                         switch (constraint.getTagNo())
                         {
                         case 0:
-                            int tmpInt = DERInteger.getInstance(constraint).getValue().intValue();
+                            int tmpInt = DERInteger.getInstance(constraint, false).getValue().intValue();
                             if (tmpInt == 0)
                             {
                                 explicitPolicy = 0;
@@ -2054,13 +2056,13 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
         Iterator crl_iter;
         try 
         {
-            Collection crl_coll = findCRLs(crlselect, paramsPKIX.getCertStores());
+            Collection crl_coll = CRL_UTIL.findCRLs(crlselect, paramsPKIX);
             crl_iter = crl_coll.iterator();
             
             if (crl_coll.isEmpty())
             {
                 // notifcation - no local crls found
-                crl_coll = findCRLs(new X509CRLStoreSelector(),paramsPKIX.getCertStores());
+                crl_coll = CRL_UTIL.findCRLs(new X509CRLStoreSelector(),paramsPKIX);
                 Iterator it = crl_coll.iterator();
                 List nonMatchingCrlNames = new ArrayList();
                 while (it.hasNext())
@@ -2221,12 +2223,13 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                     {
                         reason = crlReasons[reasonCode.getValue().intValue()];
                     }
-                    else
-                    {
-                        reason = crlReasons[7];
-                    }
                 }
-                
+
+                if (reason == null)
+                {
+                    reason = crlReasons[7]; // unknown
+                }
+
                 // i18n reason
                 LocaleString ls = new LocaleString(RESOURCE_NAME, reason);
                 
@@ -2312,7 +2315,7 @@ public class PKIXCertPathReviewer extends CertPathValidatorUtilities
                 Iterator it;
                 try 
                 {
-                    it  = findCRLs(baseSelect, paramsPKIX.getCertStores()).iterator();
+                    it  = CRL_UTIL.findCRLs(baseSelect, paramsPKIX).iterator();
                 }
                 catch (AnnotatedException ae)
                 {
