@@ -1,10 +1,10 @@
 package org.bouncycastle.cms.test;
 
+import javax.crypto.Cipher;
+
 import junit.framework.Test;
 import junit.framework.TestSuite;
 
-import javax.crypto.Cipher;
-
 public class AllTests 
 {
     public static void main (String[] args) 
@@ -17,14 +17,24 @@ public class AllTests
         throws Exception
     {   
         TestSuite suite = new TestSuite("CMS tests");
-        
-        suite.addTest(CompressedDataTest.suite());
-        suite.addTest(SignedDataTest.suite());
-        suite.addTest(EnvelopedDataTest.suite());
 
+        suite.addTest(AuthenticatedDataTest.suite());
+        suite.addTest(AuthenticatedDataStreamTest.suite());
+        suite.addTest(CompressedDataTest.suite());
+        suite.addTest(NewCompressedDataTest.suite());
+        suite.addTest(SignedDataTest.suite());
+        suite.addTest(NewSignedDataTest.suite());
+        suite.addTest(EnvelopedDataTest.suite());
+        suite.addTest(NewEnvelopedDataTest.suite());
+        suite.addTest(NewAuthenticatedDataTest.suite());
+        suite.addTest(NewAuthenticatedDataStreamTest.suite());
         suite.addTest(CompressedDataStreamTest.suite());
+        suite.addTest(NewCompressedDataStreamTest.suite());
         suite.addTest(SignedDataStreamTest.suite());
+        suite.addTest(NewSignedDataStreamTest.suite());
         suite.addTest(EnvelopedDataStreamTest.suite());
+        suite.addTest(NewEnvelopedDataStreamTest.suite());
+
         suite.addTest(MiscDataStreamTest.suite());
         suite.addTest(Rfc4134Test.suite());
 
