@@ -5,6 +5,7 @@ import java.io.IOException;
 import java.security.KeyFactory;
 import java.security.KeyPair;
 import java.security.MessageDigest;
+import java.security.NoSuchAlgorithmException;
 import java.security.cert.CertStore;
 import java.security.cert.CollectionCertStoreParameters;
 import java.security.cert.X509CRL;
@@ -22,7 +23,6 @@ import java.util.Map;
 import junit.framework.Test;
 import junit.framework.TestCase;
 import junit.framework.TestSuite;
-
 import org.bouncycastle.asn1.ASN1EncodableVector;
 import org.bouncycastle.asn1.ASN1InputStream;
 import org.bouncycastle.asn1.ASN1OctetString;
@@ -32,7 +32,9 @@ import org.bouncycastle.asn1.cms.Attribute;
 import org.bouncycastle.asn1.cms.AttributeTable;
 import org.bouncycastle.asn1.cms.CMSAttributes;
 import org.bouncycastle.asn1.cms.ContentInfo;
+import org.bouncycastle.asn1.oiw.OIWObjectIdentifiers;
 import org.bouncycastle.asn1.pkcs.PKCSObjectIdentifiers;
+import org.bouncycastle.cms.CMSConfig;
 import org.bouncycastle.cms.CMSProcessable;
 import org.bouncycastle.cms.CMSProcessableByteArray;
 import org.bouncycastle.cms.CMSSignedData;
@@ -41,8 +43,8 @@ import org.bouncycastle.cms.CMSSignedDataParser;
 import org.bouncycastle.cms.SignerId;
 import org.bouncycastle.cms.SignerInformation;
 import org.bouncycastle.cms.SignerInformationStore;
+import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.util.encoders.Base64;
-import org.bouncycastle.util.encoders.Hex;
 import org.bouncycastle.util.io.Streams;
 import org.bouncycastle.x509.X509AttributeCertificate;
 import org.bouncycastle.x509.X509CollectionStoreParameters;
@@ -51,6 +53,7 @@ import org.bouncycastle.x509.X509Store;
 public class SignedDataTest
     extends TestCase
 {
+    private static final String BC = BouncyCastleProvider.PROVIDER_NAME;
 
     boolean DEBUG = true;
 
@@ -382,6 +385,7 @@ public class SignedDataTest
         + "dmEgU29mdHdhcmUgQ29kZSBTaWduaW5nMRwwGgYDVQQDExNKQ0UgQ29kZSBT"
         + "aWduaW5nIENBAgEzMAkGBSsOAwIaBQAwCwYHKoZIzjgEAQUABC8wLQIVAIGV"
         + "khm+kbV4a/+EP45PHcq0hIViAhR4M9os6IrJnoEDS3Y3l7O6zrSosA==");
+
     /*
      *
      *  INFRASTRUCTURE
@@ -445,7 +449,7 @@ public class SignedDataTest
     private void verifySignatures(CMSSignedData s, byte[] contentDigest) 
         throws Exception
     {
-        CertStore               certStore = s.getCertificatesAndCRLs("Collection", "BC");
+        CertStore               certStore = s.getCertificatesAndCRLs("Collection", BC);
         SignerInformationStore  signers = s.getSignerInfos();
         
         Collection              c = signers.getSigners();
@@ -459,7 +463,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
     
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
             
             if (contentDigest != null)
             {
@@ -470,8 +474,8 @@ public class SignedDataTest
         Collection certColl = certStore.getCertificates(null);
         Collection crlColl = certStore.getCRLs(null);
 
-        assertEquals(certColl.size(), s.getCertificates("Collection", "BC").getMatches(null).size());
-        assertEquals(crlColl.size(), s.getCRLs("Collection", "BC").getMatches(null).size());
+        assertEquals(certColl.size(), s.getCertificates("Collection", BC).getMatches(null).size());
+        assertEquals(crlColl.size(), s.getCRLs("Collection", BC).getMatches(null).size());
     }
 
     private void verifySignatures(CMSSignedData s) 
@@ -491,7 +495,7 @@ public class SignedDataTest
         certList.add(_signCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -501,10 +505,10 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData s = gen.generate(msg, "BC");
+        CMSSignedData s = gen.generate(msg, BC);
 
-        MessageDigest sha1 = MessageDigest.getInstance("SHA1", "BC");
-        MessageDigest md5 = MessageDigest.getInstance("MD5", "BC");
+        MessageDigest sha1 = MessageDigest.getInstance("SHA1", BC);
+        MessageDigest md5 = MessageDigest.getInstance("MD5", BC);
         Map hashes = new HashMap();
         byte[] sha1Hash = sha1.digest(data);
         byte[] md5Hash = md5.digest(data);
@@ -527,7 +531,7 @@ public class SignedDataTest
         certList.add(_signCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -537,14 +541,14 @@ public class SignedDataTest
         
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData s = gen.generate(msg, true, "BC");
+        CMSSignedData s = gen.generate(msg, true, BC);
 
         ByteArrayInputStream bIn = new ByteArrayInputStream(s.getEncoded());
         ASN1InputStream      aIn = new ASN1InputStream(bIn);
         
         s = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));
 
-        certs = s.getCertificatesAndCRLs("Collection", "BC");
+        certs = s.getCertificatesAndCRLs("Collection", BC);
 
         SignerInformationStore  signers = s.getSignerInfos();
         
@@ -564,7 +568,7 @@ public class SignedDataTest
 
             sid = signer.getSID();
             
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
 
             //
             // check content digest
@@ -591,16 +595,16 @@ public class SignedDataTest
            
         gen.addSigners(s.getSignerInfos());
         
-        gen.addCertificatesAndCRLs(s.getCertificatesAndCRLs("Collection", "BC"));
+        gen.addCertificatesAndCRLs(s.getCertificatesAndCRLs("Collection", BC));
            
-        s = gen.generate(msg, true, "BC");
+        s = gen.generate(msg, true, BC);
 
         bIn = new ByteArrayInputStream(s.getEncoded());
         aIn = new ASN1InputStream(bIn);
 
         s = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));
 
-        certs = s.getCertificatesAndCRLs("Collection", "BC");
+        certs = s.getCertificatesAndCRLs("Collection", BC);
 
         signers = s.getSignerInfos();
         c = signers.getSigners();
@@ -616,7 +620,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
 
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
         }
         
         checkSignerStoreReplacement(s, signers);
@@ -632,7 +636,7 @@ public class SignedDataTest
         certList.add(_signCert);
     
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
     
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
     
@@ -640,20 +644,19 @@ public class SignedDataTest
     
         gen.addCertificatesAndCRLs(certs);
     
-        CMSSignedData s = gen.generate(CMSSignedDataGenerator.DATA, msg, false, "BC", false);
+        CMSSignedData s = gen.generate(CMSSignedDataGenerator.DATA, msg, false, BC, false);
     
         //
         // compute expected content digest
         //
-        MessageDigest md = MessageDigest.getInstance("SHA1", "BC");
+        MessageDigest md = MessageDigest.getInstance("SHA1", BC);
         
         verifySignatures(s, md.digest("Hello world!".getBytes()));
     }
 
-    public void testSHA1WithRSAAndAttributeTable()
+    public void testSHA1WithRSAViaConfig()
         throws Exception
     {
-        MessageDigest       md = MessageDigest.getInstance("SHA1", "BC");
         List                certList = new ArrayList();
         CMSProcessable      msg = new CMSProcessableByteArray("Hello world!".getBytes());
 
@@ -661,7 +664,61 @@ public class SignedDataTest
         certList.add(_signCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
+
+        // set some bogus mappings.
+        CMSConfig.setSigningEncryptionAlgorithmMapping(PKCSObjectIdentifiers.rsaEncryption.getId(), "XXXX");
+        CMSConfig.setSigningDigestAlgorithmMapping(OIWObjectIdentifiers.idSHA1.getId(), "YYYY");
+
+        CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
+
+        gen.addSigner(_origKP.getPrivate(), _origCert, CMSSignedDataGenerator.DIGEST_SHA1);
+
+        gen.addCertificatesAndCRLs(certs);
+
+        CMSSignedData s;
+
+        try
+        {
+            // try the bogus mappings
+            s = gen.generate(CMSSignedDataGenerator.DATA, msg, false, BC, false);
+        }
+        catch (NoSuchAlgorithmException e)
+        {
+            if (!e.getMessage().startsWith("no such algorithm: YYYYwithXXXX"))
+            {
+                throw e;
+            }
+        }
+        finally
+        {
+            // reset to the real ones
+            CMSConfig.setSigningEncryptionAlgorithmMapping(PKCSObjectIdentifiers.rsaEncryption.getId(), "RSA");
+            CMSConfig.setSigningDigestAlgorithmMapping(OIWObjectIdentifiers.idSHA1.getId(), "SHA1"); 
+        }
+
+        s = gen.generate(CMSSignedDataGenerator.DATA, msg, false, BC, false);
+
+        //
+        // compute expected content digest
+        //
+        MessageDigest md = MessageDigest.getInstance("SHA1", BC);
+
+        verifySignatures(s, md.digest("Hello world!".getBytes()));
+    }
+
+    public void testSHA1WithRSAAndAttributeTable()
+        throws Exception
+    {
+        MessageDigest       md = MessageDigest.getInstance("SHA1", BC);
+        List                certList = new ArrayList();
+        CMSProcessable      msg = new CMSProcessableByteArray("Hello world!".getBytes());
+
+        certList.add(_origCert);
+        certList.add(_signCert);
+
+        CertStore           certs = CertStore.getInstance("Collection",
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -679,7 +736,7 @@ public class SignedDataTest
         gen.addCertificatesAndCRLs(certs);
 
 
-        CMSSignedData s = gen.generate(CMSSignedDataGenerator.DATA, null, false, "BC");
+        CMSSignedData s = gen.generate(CMSSignedDataGenerator.DATA, null, false, BC);
 
         //
         // the signature is detached, so need to add msg before passing on
@@ -799,7 +856,7 @@ public class SignedDataTest
     {
         X509EncodedKeySpec  pubSpec = new X509EncodedKeySpec(_signEcDsaKP.getPublic().getEncoded());
         PKCS8EncodedKeySpec privSpec = new PKCS8EncodedKeySpec(_signEcDsaKP.getPrivate().getEncoded());
-        KeyFactory          keyFact = KeyFactory.getInstance("EC", "BC");
+        KeyFactory          keyFact = KeyFactory.getInstance("EC", BC);
         KeyPair             kp = new KeyPair(keyFact.generatePublic(pubSpec), keyFact.generatePrivate(privSpec));
         
         encapsulatedTest(kp, _signEcDsaCert, CMSSignedDataGenerator.DIGEST_SHA512);
@@ -841,7 +898,7 @@ public class SignedDataTest
         certList.add(_signCrl);
 
         CertStore           certsAndCrls = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -849,10 +906,10 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certsAndCrls);
 
-        CMSSignedData s = gen.generate(msg, true, "BC");
+        CMSSignedData s = gen.generate(msg, true, BC);
         SignerInformation origSigner = (SignerInformation)s.getSignerInfos().getSigners().toArray()[0];
-        SignerInformationStore counterSigners1 = gen.generateCounterSigners(origSigner, "BC");
-        SignerInformationStore counterSigners2 = gen.generateCounterSigners(origSigner, "BC");
+        SignerInformationStore counterSigners1 = gen.generateCounterSigners(origSigner, BC);
+        SignerInformationStore counterSigners2 = gen.generateCounterSigners(origSigner, BC);
 
         SignerInformation signer1 = SignerInformation.addCounterSigners(origSigner, counterSigners1);
         SignerInformation signer2 = SignerInformation.addCounterSigners(signer1, counterSigners2);
@@ -871,7 +928,7 @@ public class SignedDataTest
             X509Certificate cert = (X509Certificate)certIt.next();
 
             assertNull(cSigner.getSignedAttributes().get(PKCSObjectIdentifiers.pkcs_9_at_contentType));
-            assertEquals(true, cSigner.verify(cert, "BC"));
+            assertEquals(true, cSigner.verify(cert, BC));
         }
     }
 
@@ -885,7 +942,7 @@ public class SignedDataTest
         certList.add(_signCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -893,12 +950,12 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData s = gen.generate(CMSSignedDataGenerator.DATA, msg, false, "BC", false);
+        CMSSignedData s = gen.generate(CMSSignedDataGenerator.DATA, msg, false, BC, false);
 
         //
         // compute expected content digest
         //
-        MessageDigest md = MessageDigest.getInstance(digestName, "BC");
+        MessageDigest md = MessageDigest.getInstance(digestName, BC);
 
         verifySignatures(s, md.digest("Hello world!".getBytes()));
     }
@@ -918,7 +975,7 @@ public class SignedDataTest
         certList.add(_signCrl);
 
         CertStore           certsAndCrls = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -926,14 +983,16 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certsAndCrls);
 
-        CMSSignedData s = gen.generate(msg, true, "BC");
+        CMSSignedData s = gen.generate(msg, true, BC);
 
+        assertEquals(3, s.getVersion());
+        
         ByteArrayInputStream bIn = new ByteArrayInputStream(s.getEncoded());
         ASN1InputStream      aIn = new ASN1InputStream(bIn);
 
         s = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));
 
-        certsAndCrls = s.getCertificatesAndCRLs("Collection", "BC");
+        certsAndCrls = s.getCertificatesAndCRLs("Collection", BC);
 
         SignerInformationStore  signers = s.getSignerInfos();
         Collection              c = signers.getSigners();
@@ -947,7 +1006,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
 
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
         }
 
         //
@@ -967,16 +1026,16 @@ public class SignedDataTest
 
         gen.addSigners(s.getSignerInfos());
 
-        gen.addCertificatesAndCRLs(s.getCertificatesAndCRLs("Collection", "BC"));
+        gen.addCertificatesAndCRLs(s.getCertificatesAndCRLs("Collection", BC));
 
-        s = gen.generate(msg, true, "BC");
+        s = gen.generate(msg, true, BC);
 
         bIn = new ByteArrayInputStream(s.getEncoded());
         aIn = new ASN1InputStream(bIn);
 
         s = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));
 
-        certsAndCrls = s.getCertificatesAndCRLs("Collection", "BC");
+        certsAndCrls = s.getCertificatesAndCRLs("Collection", BC);
 
         signers = s.getSignerInfos();
         c = signers.getSigners();
@@ -990,7 +1049,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
 
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
         }
 
         checkSignerStoreReplacement(s, signers);
@@ -1011,7 +1070,7 @@ public class SignedDataTest
         certList.add(_signCrl);
 
         CertStore           certsAndCrls = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
     
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
     
@@ -1019,14 +1078,14 @@ public class SignedDataTest
     
         gen.addCertificatesAndCRLs(certsAndCrls);
     
-        CMSSignedData s = gen.generate(msg, true, "BC");
+        CMSSignedData s = gen.generate(msg, true, BC);
     
         ByteArrayInputStream bIn = new ByteArrayInputStream(s.getEncoded());
         ASN1InputStream      aIn = new ASN1InputStream(bIn);
         
         s = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));
     
-        certsAndCrls = s.getCertificatesAndCRLs("Collection", "BC");
+        certsAndCrls = s.getCertificatesAndCRLs("Collection", BC);
     
         SignerInformationStore  signers = s.getSignerInfos();
         Collection              c = signers.getSigners();
@@ -1040,7 +1099,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
     
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
         }
 
         //
@@ -1060,16 +1119,16 @@ public class SignedDataTest
            
         gen.addSigners(s.getSignerInfos());
         
-        gen.addCertificatesAndCRLs(s.getCertificatesAndCRLs("Collection", "BC"));
+        gen.addCertificatesAndCRLs(s.getCertificatesAndCRLs("Collection", BC));
            
-        s = gen.generate(msg, true, "BC");
+        s = gen.generate(msg, true, BC);
     
         bIn = new ByteArrayInputStream(s.getEncoded());
         aIn = new ASN1InputStream(bIn);
     
         s = new CMSSignedData(ContentInfo.getInstance(aIn.readObject()));
     
-        certsAndCrls = s.getCertificatesAndCRLs("Collection", "BC");
+        certsAndCrls = s.getCertificatesAndCRLs("Collection", BC);
     
         signers = s.getSignerInfos();
         c = signers.getSigners();
@@ -1083,7 +1142,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
     
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
         }
         
         checkSignerStoreReplacement(s, signers);
@@ -1099,7 +1158,7 @@ public class SignedDataTest
     {
         CMSSignedData s = CMSSignedData.replaceSigners(orig, signers);
         
-        CertStore certs = s.getCertificatesAndCRLs("Collection", "BC");
+        CertStore certs = s.getCertificatesAndCRLs("Collection", BC);
         
         signers = s.getSignerInfos();
         Collection c = signers.getSigners();
@@ -1113,7 +1172,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
     
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
         }
     }
     
@@ -1122,7 +1181,7 @@ public class SignedDataTest
     {
         CMSSignedData s = new CMSSignedData(new CMSProcessableByteArray(disorderedMessage), disorderedSet);
 
-        CertStore certs = s.getCertificatesAndCRLs("Collection", "BC");
+        CertStore certs = s.getCertificatesAndCRLs("Collection", BC);
 
         SignerInformationStore  signers = s.getSignerInfos();
         Collection              c = signers.getSigners();
@@ -1136,7 +1195,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
 
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
         }
     }
     
@@ -1149,7 +1208,7 @@ public class SignedDataTest
         certList.add(_signCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -1157,7 +1216,7 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData s = gen.generate(null, false, "BC");
+        CMSSignedData s = gen.generate(null, false, BC);
 
         ByteArrayInputStream bIn = new ByteArrayInputStream(s.getEncoded());
         ASN1InputStream      aIn = new ASN1InputStream(bIn);
@@ -1177,7 +1236,7 @@ public class SignedDataTest
         certList.add(_signDsaCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -1188,15 +1247,15 @@ public class SignedDataTest
         X509AttributeCertificate attrCert = CMSTestUtil.getAttributeCertificate();
 
         X509Store store = X509Store.getInstance("AttributeCertificate/Collection",
-                                    new X509CollectionStoreParameters(Collections.singleton(attrCert)), "BC");
+                                    new X509CollectionStoreParameters(Collections.singleton(attrCert)), BC);
 
         gen.addAttributeCertificates(store);
 
-        CMSSignedData sd = gen.generate(msg, "BC");
+        CMSSignedData sd = gen.generate(msg, BC);
 
         assertEquals(4, sd.getVersion());
 
-        store = sd.getAttributeCertificates("Collection", "BC");
+        store = sd.getAttributeCertificates("Collection", BC);
 
         Collection coll = store.getMatches(null);
 
@@ -1212,7 +1271,7 @@ public class SignedDataTest
         certList.add(_signCert);
 
         certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
 
         //
@@ -1233,7 +1292,7 @@ public class SignedDataTest
         certList.add(_signDsaCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -1241,7 +1300,7 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData sd = gen.generate(msg, "BC");
+        CMSSignedData sd = gen.generate(msg, BC);
 
         //
         // create new certstore
@@ -1251,7 +1310,7 @@ public class SignedDataTest
         certList.add(_signCert);
 
         certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         //
         // replace certs
@@ -1271,7 +1330,7 @@ public class SignedDataTest
         certList.add(_signDsaCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -1279,7 +1338,7 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData sd = gen.generate(msg, true, "BC");
+        CMSSignedData sd = gen.generate(msg, true, BC);
 
         //
         // create new certstore
@@ -1289,7 +1348,7 @@ public class SignedDataTest
         certList.add(_signCert);
 
         certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         //
         // replace certs
@@ -1310,7 +1369,7 @@ public class SignedDataTest
         certList.add(_signDsaCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -1318,9 +1377,9 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData sd = gen.generate(msg, true, "BC");
+        CMSSignedData sd = gen.generate(msg, true, BC);
 
-        certs = sd.getCertificatesAndCRLs("Collection", "BC");
+        certs = sd.getCertificatesAndCRLs("Collection", BC);
         Iterator it = certs.getCertificates(null).iterator();
 
         assertEquals(_origCert, it.next());
@@ -1339,7 +1398,7 @@ public class SignedDataTest
         certList.add(_origCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -1347,9 +1406,9 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData sd = gen.generate(msg, true, "BC");
+        CMSSignedData sd = gen.generate(msg, true, BC);
 
-        certs = sd.getCertificatesAndCRLs("Collection", "BC");
+        certs = sd.getCertificatesAndCRLs("Collection", BC);
         Iterator it = certs.getCertificates(null).iterator();
 
         assertEquals(_signCert, it.next());
@@ -1367,7 +1426,7 @@ public class SignedDataTest
         certList.add(_signCert);
 
         CertStore           certs = CertStore.getInstance("Collection",
-                        new CollectionCertStoreParameters(certList), "BC");
+                        new CollectionCertStoreParameters(certList), BC);
 
         CMSSignedDataGenerator gen = new CMSSignedDataGenerator();
 
@@ -1375,7 +1434,7 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData original = gen.generate(msg, true, "BC");
+        CMSSignedData original = gen.generate(msg, true, BC);
 
         //
         // create new Signer
@@ -1386,7 +1445,7 @@ public class SignedDataTest
 
         gen.addCertificatesAndCRLs(certs);
 
-        CMSSignedData newSD = gen.generate(msg, true, "BC");
+        CMSSignedData newSD = gen.generate(msg, true, BC);
 
         //
         // replace signer
@@ -1422,6 +1481,34 @@ public class SignedDataTest
         testSample("PSSSignData.data", "PSSSignDataSHA512.sig");
     }
 
+    public void testCounterSig()
+        throws Exception
+    {
+        CMSSignedData sig = new CMSSignedData(getInput("counterSig.p7m"));
+
+        SignerInformationStore ss = sig.getSignerInfos();
+        Collection signers = ss.getSigners();
+
+        SignerInformationStore cs = ((SignerInformation)signers.iterator().next()).getCounterSignatures();
+        Collection csSigners = cs.getSigners();
+        assertEquals(1, csSigners.size());
+
+        Iterator it = csSigners.iterator();
+        while (it.hasNext())
+        {
+            SignerInformation   cSigner = (SignerInformation)it.next();
+            Collection          certCollection = sig.getCertificatesAndCRLs("Collection", BC).getCertificates(cSigner.getSID());
+
+            Iterator        certIt = certCollection.iterator();
+            X509Certificate cert = (X509Certificate)certIt.next();
+
+            assertNull(cSigner.getSignedAttributes().get(PKCSObjectIdentifiers.pkcs_9_at_contentType));
+            assertEquals(true, cSigner.verify(cert, BC));
+        }
+        
+        verifySignatures(sig);
+    }
+
     private void testSample(String sigName)
         throws Exception
     {
@@ -1463,7 +1550,7 @@ public class SignedDataTest
     private void verifySignatures(CMSSignedDataParser sp)
         throws Exception
     {
-        CertStore               certs = sp.getCertificatesAndCRLs("Collection", "BC");
+        CertStore               certs = sp.getCertificatesAndCRLs("Collection", BC);
         SignerInformationStore  signers = sp.getSignerInfos();
 
         Collection              c = signers.getSigners();
@@ -1477,7 +1564,7 @@ public class SignedDataTest
             Iterator        certIt = certCollection.iterator();
             X509Certificate cert = (X509Certificate)certIt.next();
 
-            assertEquals(true, signer.verify(cert, "BC"));
+            assertEquals(true, signer.verify(cert, BC));
         }
     }
 }
