@@ -1,11 +1,11 @@
 package org.bouncycastle.jce.provider.test;
 
+import java.security.Security;
+
 import org.bouncycastle.jce.provider.BouncyCastleProvider;
 import org.bouncycastle.util.test.Test;
 import org.bouncycastle.util.test.TestResult;
 
-import java.security.Security;
-
 public class RegressionTest
 {
     public static Test[]    tests = {
@@ -34,7 +34,6 @@ public class RegressionTest
         new AttrCertTest(),
         new CertTest(),
         new PKCS10CertRequestTest(),
-        new PKCS7SignedDataTest(),
         new EncryptedPrivateKeyInfoTest(),
         new KeyStoreTest(),
         new PKCS12StoreTest(),
@@ -64,6 +63,8 @@ public class RegressionTest
         new AttrCertSelectorTest(),
         new SerialisationTest(),
         new SigNameTest(),
+        new MQVTest(),
+        new CMacTest(),
         new CRL5Test()
     };
 
